@@ -1,34 +1,25 @@
 # mysql/mysqlconnector.py
-# Copyright (C) 2005-2012 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-"""Support for the MySQL database via the MySQL Connector/Python adapter.
+"""
+.. dialect:: mysql+mysqlconnector
+    :name: MySQL Connector/Python
+    :dbapi: myconnpy
+    :connectstring: mysql+mysqlconnector://<user>:<password>@<host>[:<port>]/<dbname>
+    :url: http://dev.mysql.com/downloads/connector/python/
 
-MySQL Connector/Python is available at:
-
-    https://launchpad.net/myconnpy
-
-Connecting
------------
-
-Connect string format::
-
-    mysql+mysqlconnector://<user>:<password>@<host>[:<port>]/<dbname>
 
 """
 
-import re
-
-from sqlalchemy.dialects.mysql.base import (MySQLDialect,
+from .base import (MySQLDialect,
     MySQLExecutionContext, MySQLCompiler, MySQLIdentifierPreparer,
     BIT)
 
-from sqlalchemy.engine import base as engine_base, default
-from sqlalchemy.sql import operators as sql_operators
-from sqlalchemy import exc, log, schema, sql, types as sqltypes, util
-from sqlalchemy import processors
+from ... import util
+
 
 class MySQLExecutionContext_mysqlconnector(MySQLExecutionContext):
 
@@ -37,24 +28,28 @@ class MySQLExecutionContext_mysqlconnector(MySQLExecutionContext):
 
 
 class MySQLCompiler_mysqlconnector(MySQLCompiler):
-    def visit_mod(self, binary, **kw):
-        return self.process(binary.left) + " %% " + self.process(binary.right)
+    def visit_mod_binary(self, binary, operator, **kw):
+        return self.process(binary.left, **kw) + " %% " + \
+                        self.process(binary.right, **kw)
 
     def post_process_text(self, text):
         return text.replace('%', '%%')
 
+
 class MySQLIdentifierPreparer_mysqlconnector(MySQLIdentifierPreparer):
 
     def _escape_identifier(self, value):
         value = value.replace(self.escape_quote, self.escape_to_quote)
         return value.replace("%", "%%")
 
+
 class _myconnpyBIT(BIT):
     def result_processor(self, dialect, coltype):
         """MySQL-connector already converts mysql bits, so."""
 
         return None
 
+
 class MySQLDialect_mysqlconnector(MySQLDialect):
     driver = 'mysqlconnector'
     supports_unicode_statements = True
@@ -84,12 +79,13 @@ class MySQLDialect_mysqlconnector(MySQLDialect):
 
     def create_connect_args(self, url):
         opts = url.translate_connect_args(username='user')
+
         opts.update(url.query)
 
         util.coerce_kw_type(opts, 'buffered', bool)
         util.coerce_kw_type(opts, 'raise_on_warnings', bool)
-        opts['buffered'] = True
-        opts['raise_on_warnings'] = True
+        opts.setdefault('buffered', True)
+        opts.setdefault('raise_on_warnings', True)
 
         # FOUND_ROWS must be set in ClientFlag to enable
         # supports_sane_rowcount.
@@ -105,24 +101,21 @@ class MySQLDialect_mysqlconnector(MySQLDialect):
 
     def _get_server_version_info(self, connection):
         dbapi_con = connection.connection
-
-        from mysql.connector.constants import ClientFlag
-        dbapi_con.set_client_flag(ClientFlag.FOUND_ROWS)
-
         version = dbapi_con.get_server_version()
         return tuple(version)
 
     def _detect_charset(self, connection):
-        return connection.connection.get_characterset_info()
+        return connection.connection.charset
 
     def _extract_error_code(self, exception):
         return exception.errno
 
-    def is_disconnect(self, e):
+    def is_disconnect(self, e, connection, cursor):
         errnos = (2006, 2013, 2014, 2045, 2055, 2048)
-        exceptions = (self.dbapi.OperationalError,self.dbapi.InterfaceError)
+        exceptions = (self.dbapi.OperationalError, self.dbapi.InterfaceError)
         if isinstance(e, exceptions):
-            return e.errno in errnos
+            return e.errno in errnos or \
+                "MySQL Connection not available." in str(e)
         else:
             return False
 
