@@ -346,7 +346,7 @@ class PGDialect_psycopg2(PGDialect):
     supports_unicode_statements = False
     # end Py2K
     default_paramstyle = 'pyformat'
-    supports_sane_multi_rowcount = False
+    supports_sane_multi_rowcount = False  # set to true based on psycopg2 version
     execution_ctx_cls = PGExecutionContext_psycopg2
     statement_compiler = PGCompiler_psycopg2
     preparer = PGIdentifierPreparer_psycopg2
@@ -390,6 +390,9 @@ class PGDialect_psycopg2(PGDialect):
                         self._hstore_oids(connection.connection) \
                             is not None
 
+        # http://initd.org/psycopg/docs/news.html#what-s-new-in-psycopg-2-0-9
+        self.supports_sane_multi_rowcount = self.psycopg2_version >= (2, 0, 9)
+
     @classmethod
     def dbapi(cls):
         import psycopg2
