@@ -1,5 +1,5 @@
 # engine/default.py
-# Copyright (C) 2005-2012 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -12,10 +12,11 @@ as the base class for their own corresponding classes.
 
 """
 
-import re, random
-from sqlalchemy.engine import base, reflection
-from sqlalchemy.sql import compiler, expression
-from sqlalchemy import exc, types as sqltypes, util, pool, processors
+import re
+import random
+from . import reflection, interfaces, result
+from ..sql import compiler, expression
+from .. import exc, types as sqltypes, util, pool, processors
 import codecs
 import weakref
 
@@ -24,7 +25,7 @@ AUTOCOMMIT_REGEXP = re.compile(
             re.I | re.UNICODE)
 
 
-class DefaultDialect(base.Dialect):
+class DefaultDialect(interfaces.Dialect):
     """Default implementation of Dialect"""
 
     statement_compiler = compiler.SQLCompiler
@@ -33,6 +34,10 @@ class DefaultDialect(base.Dialect):
     preparer = compiler.IdentifierPreparer
     supports_alter = True
 
+    # the first value we'd get for an autoincrement
+    # column.
+    default_sequence_base = 1
+
     # most DBAPIs happy with this for execute().
     # not cx_oracle.
     execute_sequence_format = tuple
@@ -44,7 +49,6 @@ class DefaultDialect(base.Dialect):
     postfetch_lastrowid = True
     implicit_returning = False
 
-
     supports_native_enum = False
     supports_native_boolean = False
 
@@ -65,7 +69,6 @@ class DefaultDialect(base.Dialect):
     description_encoding = 'use_encoding'
     # end Py2K
 
-
     name = 'default'
 
     # length at which to truncate
@@ -86,10 +89,11 @@ class DefaultDialect(base.Dialect):
     default_paramstyle = 'named'
     supports_default_values = False
     supports_empty_insert = True
+    supports_multivalues_insert = False
 
     server_version_info = None
 
-    # indicates symbol names are 
+    # indicates symbol names are
     # UPPERCASEd if they are case insensitive
     # within the database.
     # if this is True, the methods normalize_name()
@@ -98,27 +102,18 @@ class DefaultDialect(base.Dialect):
 
     reflection_options = ()
 
-    def __init__(self, convert_unicode=False, assert_unicode=False,
+    def __init__(self, convert_unicode=False,
                  encoding='utf-8', paramstyle=None, dbapi=None,
                  implicit_returning=None,
+                 case_sensitive=True,
                  label_length=None, **kwargs):
 
         if not getattr(self, 'ported_sqla_06', True):
             util.warn(
-                "The %s dialect is not yet ported to SQLAlchemy 0.6/0.7" %
+                "The %s dialect is not yet ported to the 0.6 format" %
                 self.name)
 
         self.convert_unicode = convert_unicode
-        if assert_unicode:
-            util.warn_deprecated(
-                "assert_unicode is deprecated. "
-                "SQLAlchemy emits a warning in all cases where it "
-                "would otherwise like to encode a Python unicode object "
-                "into a specific encoding but a plain bytestring is "
-                "received. "
-                "This does *not* apply to DBAPIs that coerce Unicode "
-                "natively.")
-
         self.encoding = encoding
         self.positional = False
         self._ischema = None
@@ -135,6 +130,8 @@ class DefaultDialect(base.Dialect):
         self.identifier_preparer = self.preparer(self)
         self.type_compiler = self.type_compiler(self)
 
+        self.case_sensitive = case_sensitive
+
         if label_length and label_length > self.max_identifier_length:
             raise exc.ArgumentError(
                     "Label length of %d is greater than this dialect's"
@@ -143,11 +140,13 @@ class DefaultDialect(base.Dialect):
         self.label_length = label_length
 
         if self.description_encoding == 'use_encoding':
-            self._description_decoder = processors.to_unicode_processor_factory(
+            self._description_decoder = \
+                            processors.to_unicode_processor_factory(
                                             encoding
                                     )
         elif self.description_encoding is not None:
-            self._description_decoder = processors.to_unicode_processor_factory(
+            self._description_decoder = \
+                            processors.to_unicode_processor_factory(
                                             self.description_encoding
                                     )
         self._encoder = codecs.getencoder(self.encoding)
@@ -186,7 +185,7 @@ class DefaultDialect(base.Dialect):
         self.returns_unicode_strings = self._check_unicode_returns(connection)
 
         self.do_rollback(connection.connection)
- 
+
     def on_connect(self):
         """return a callable which sets up a newly created DBAPI connection.
 
@@ -210,16 +209,18 @@ class DefaultDialect(base.Dialect):
         # end Py2K
         # Py3K
         #cast_to = str
+
         def check_unicode(formatstr, type_):
             cursor = connection.connection.cursor()
             try:
                 try:
                     cursor.execute(
                         cast_to(
-                            expression.select( 
+                            expression.select(
                             [expression.cast(
                                 expression.literal_column(
-                                        "'test %s returns'" % formatstr), type_)
+                                        "'test %s returns'" % formatstr),
+                                        type_)
                             ]).compile(dialect=self)
                         )
                     )
@@ -245,17 +246,18 @@ class DefaultDialect(base.Dialect):
             return unicode_for_varchar
 
     def type_descriptor(self, typeobj):
-        """Provide a database-specific ``TypeEngine`` object, given
+        """Provide a database-specific :class:`.TypeEngine` object, given
         the generic object which comes from the types module.
 
         This method looks for a dictionary called
         ``colspecs`` as a class or instance-level variable,
-        and passes on to ``types.adapt_type()``.
+        and passes on to :func:`.types.adapt_type`.
 
         """
         return sqltypes.adapt_type(typeobj, self.colspecs)
 
-    def reflecttable(self, connection, table, include_columns, exclude_columns=None):
+    def reflecttable(self, connection, table, include_columns,
+                    exclude_columns=None):
         insp = reflection.Inspector.from_engine(connection)
         return insp.reflecttable(table, include_columns, exclude_columns)
 
@@ -266,14 +268,14 @@ class DefaultDialect(base.Dialect):
         """
         return {
             'constrained_columns':
-                        self.get_primary_keys(conn, table_name, 
+                        self.get_primary_keys(conn, table_name,
                                                 schema=schema, **kw)
         }
 
     def validate_identifier(self, ident):
         if len(ident) > self.max_identifier_length:
             raise exc.IdentifierError(
-                "Identifier '%s' exceeds maximum length of %d characters" % 
+                "Identifier '%s' exceeds maximum length of %d characters" %
                 (ident, self.max_identifier_length)
             )
 
@@ -285,26 +287,17 @@ class DefaultDialect(base.Dialect):
         opts.update(url.query)
         return [[], opts]
 
-    def do_begin(self, connection):
-        """Implementations might want to put logic here for turning
-        autocommit on/off, etc.
-        """
-
+    def do_begin(self, dbapi_connection):
         pass
 
-    def do_rollback(self, connection):
-        """Implementations might want to put logic here for turning
-        autocommit on/off, etc.
-        """
+    def do_rollback(self, dbapi_connection):
+        dbapi_connection.rollback()
 
-        connection.rollback()
+    def do_commit(self, dbapi_connection):
+        dbapi_connection.commit()
 
-    def do_commit(self, connection):
-        """Implementations might want to put logic here for turning
-        autocommit on/off, etc.
-        """
-
-        connection.commit()
+    def do_close(self, dbapi_connection):
+        dbapi_connection.close()
 
     def create_xid(self):
         """Create a random two-phase transaction ID.
@@ -337,12 +330,13 @@ class DefaultDialect(base.Dialect):
         return False
 
     def reset_isolation_level(self, dbapi_conn):
-        # default_isolation_level is read from the first connection 
-        # after the initial set of 'isolation_level', if any, so is 
+        # default_isolation_level is read from the first connection
+        # after the initial set of 'isolation_level', if any, so is
         # the configured default of this dialect.
         self.set_isolation_level(dbapi_conn, self.default_isolation_level)
 
-class DefaultExecutionContext(base.ExecutionContext):
+
+class DefaultExecutionContext(interfaces.ExecutionContext):
     isinsert = False
     isupdate = False
     isdelete = False
@@ -351,10 +345,12 @@ class DefaultExecutionContext(base.ExecutionContext):
     result_map = None
     compiled = None
     statement = None
+    postfetch_cols = None
+    prefetch_cols = None
     _is_implicit_returning = False
     _is_explicit_returning = False
 
-    # a hook for SQLite's translation of 
+    # a hook for SQLite's translation of
     # result column names
     _translate_colname = None
 
@@ -393,7 +389,8 @@ class DefaultExecutionContext(base.ExecutionContext):
         return self
 
     @classmethod
-    def _init_compiled(cls, dialect, connection, dbapi_connection, compiled, parameters):
+    def _init_compiled(cls, dialect, connection, dbapi_connection,
+                    compiled, parameters):
         """Initialize execution context for a Compiled construct."""
 
         self = cls.__new__(cls)
@@ -419,7 +416,8 @@ class DefaultExecutionContext(base.ExecutionContext):
 
         self.unicode_statement = unicode(compiled)
         if not dialect.supports_unicode_statements:
-            self.statement = self.unicode_statement.encode(self.dialect.encoding)
+            self.statement = self.unicode_statement.encode(
+                                        self.dialect.encoding)
         else:
             self.statement = self.unicode_statement
 
@@ -428,16 +426,16 @@ class DefaultExecutionContext(base.ExecutionContext):
         self.isdelete = compiled.isdelete
 
         if self.isinsert or self.isupdate or self.isdelete:
-            self._is_explicit_returning = compiled.statement._returning
-            self._is_implicit_returning = compiled.returning and \
-                                            not compiled.statement._returning
+            self._is_explicit_returning = bool(compiled.statement._returning)
+            self._is_implicit_returning = bool(compiled.returning and \
+                                            not compiled.statement._returning)
 
         if not parameters:
             self.compiled_parameters = [compiled.construct_params()]
         else:
             self.compiled_parameters = \
                         [compiled.construct_params(m, _group_number=grp) for
-                                        grp,m in enumerate(parameters)]
+                                        grp, m in enumerate(parameters)]
 
             self.executemany = len(parameters) > 1
 
@@ -449,8 +447,8 @@ class DefaultExecutionContext(base.ExecutionContext):
 
         processors = compiled._bind_processors
 
-        # Convert the dictionary of bind parameter values 
-        # into a dict or list to be sent to the DBAPI's 
+        # Convert the dictionary of bind parameter values
+        # into a dict or list to be sent to the DBAPI's
         # execute() or executemany() method.
         parameters = []
         if dialect.positional:
@@ -472,7 +470,8 @@ class DefaultExecutionContext(base.ExecutionContext):
                             param[dialect._encoder(key)[0]] = \
                                         processors[key](compiled_params[key])
                         else:
-                            param[dialect._encoder(key)[0]] = compiled_params[key]
+                            param[dialect._encoder(key)[0]] = \
+                                    compiled_params[key]
                 else:
                     for key in compiled_params:
                         if key in processors:
@@ -485,7 +484,8 @@ class DefaultExecutionContext(base.ExecutionContext):
         return self
 
     @classmethod
-    def _init_statement(cls, dialect, connection, dbapi_connection, statement, parameters):
+    def _init_statement(cls, dialect, connection, dbapi_connection,
+                                                    statement, parameters):
         """Initialize execution context for a string SQL statement."""
 
         self = cls.__new__(cls)
@@ -508,17 +508,18 @@ class DefaultExecutionContext(base.ExecutionContext):
             if dialect.supports_unicode_statements:
                 self.parameters = parameters
             else:
-                self.parameters= [
+                self.parameters = [
                             dict((dialect._encoder(k)[0], d[k]) for k in d)
                             for d in parameters
                         ] or [{}]
         else:
-            self.parameters = [dialect.execute_sequence_format(p) 
+            self.parameters = [dialect.execute_sequence_format(p)
                                     for p in parameters]
 
         self.executemany = len(parameters) > 1
 
-        if not dialect.supports_unicode_statements and isinstance(statement, unicode):
+        if not dialect.supports_unicode_statements and \
+            isinstance(statement, unicode):
             self.unicode_statement = statement
             self.statement = dialect._encoder(statement)[0]
         else:
@@ -550,10 +551,10 @@ class DefaultExecutionContext(base.ExecutionContext):
 
     @util.memoized_property
     def should_autocommit(self):
-        autocommit = self.execution_options.get('autocommit', 
-                                                not self.compiled and 
+        autocommit = self.execution_options.get('autocommit',
+                                                not self.compiled and
                                                 self.statement and
-                                                expression.PARSE_AUTOCOMMIT 
+                                                expression.PARSE_AUTOCOMMIT
                                                 or False)
 
         if autocommit is expression.PARSE_AUTOCOMMIT:
@@ -581,12 +582,12 @@ class DefaultExecutionContext(base.ExecutionContext):
         else:
             default_params = {}
 
-        conn._cursor_execute(self.cursor, stmt, default_params)
+        conn._cursor_execute(self.cursor, stmt, default_params, context=self)
         r = self.cursor.fetchone()[0]
         if type_ is not None:
             # apply type post processors to the result
             proc = type_._cached_result_processor(
-                        self.dialect, 
+                        self.dialect,
                         self.cursor.description[0][1]
                     )
             if proc:
@@ -609,6 +610,16 @@ class DefaultExecutionContext(base.ExecutionContext):
     def post_exec(self):
         pass
 
+    def get_result_processor(self, type_, colname, coltype):
+        """Return a 'result processor' for a given type as present in
+        cursor.description.
+
+        This has a default implementation that dialects can override
+        for context-sensitive result type handling.
+
+        """
+        return type_._cached_result_processor(self.dialect, coltype)
+
     def get_lastrowid(self):
         """return self.cursor.lastrowid, or equivalent, after an INSERT.
 
@@ -623,7 +634,7 @@ class DefaultExecutionContext(base.ExecutionContext):
         and when no explicit id value was bound to the
         statement.
 
-        The function is called once, directly after 
+        The function is called once, directly after
         post_exec() and before the transaction is committed
         or ResultProxy is generated.   If the post_exec()
         method assigns a value to `self._lastrowid`, the
@@ -641,7 +652,7 @@ class DefaultExecutionContext(base.ExecutionContext):
         pass
 
     def get_result_proxy(self):
-        return base.ResultProxy(self)
+        return result.ResultProxy(self)
 
     @property
     def rowcount(self):
@@ -655,24 +666,26 @@ class DefaultExecutionContext(base.ExecutionContext):
 
     def post_insert(self):
         if not self._is_implicit_returning and \
+            not self._is_explicit_returning and \
+            not self.compiled.inline and \
             self.dialect.postfetch_lastrowid and \
             (not self.inserted_primary_key or \
                         None in self.inserted_primary_key):
 
             table = self.compiled.statement.table
             lastrowid = self.get_lastrowid()
-
             autoinc_col = table._autoincrement_column
             if autoinc_col is not None:
                 # apply type post processors to the lastrowid
-                proc = autoinc_col.type._cached_result_processor(self.dialect, None)
+                proc = autoinc_col.type._cached_result_processor(
+                                        self.dialect, None)
                 if proc is not None:
                     lastrowid = proc(lastrowid)
 
             self.inserted_primary_key = [
-                c is autoinc_col and lastrowid or v
+                lastrowid if c is autoinc_col else v
                 for c, v in zip(
-                                    table.primary_key, 
+                                    table.primary_key,
                                     self.inserted_primary_key)
             ]
 
@@ -698,7 +711,7 @@ class DefaultExecutionContext(base.ExecutionContext):
         style of ``setinputsizes()`` on the cursor, using DB-API types
         from the bind parameter's ``TypeEngine`` objects.
 
-        This method only called by those dialects which require it, 
+        This method only called by those dialects which require it,
         currently cx_oracle.
 
         """
@@ -714,28 +727,34 @@ class DefaultExecutionContext(base.ExecutionContext):
             inputsizes = []
             for key in self.compiled.positiontup:
                 typeengine = types[key]
-                dbtype = typeengine.dialect_impl(self.dialect).get_dbapi_type(self.dialect.dbapi)
-                if dbtype is not None and (not exclude_types or dbtype not in exclude_types):
+                dbtype = typeengine.dialect_impl(self.dialect).\
+                                    get_dbapi_type(self.dialect.dbapi)
+                if dbtype is not None and \
+                    (not exclude_types or dbtype not in exclude_types):
                     inputsizes.append(dbtype)
             try:
                 self.cursor.setinputsizes(*inputsizes)
             except Exception, e:
-                self.root_connection._handle_dbapi_exception(e, None, None, None, self)
-                raise
+                self.root_connection._handle_dbapi_exception(
+                                e, None, None, None, self)
         else:
             inputsizes = {}
             for key in self.compiled.bind_names.values():
                 typeengine = types[key]
-                dbtype = typeengine.dialect_impl(self.dialect).get_dbapi_type(self.dialect.dbapi)
-                if dbtype is not None and (not exclude_types or dbtype not in exclude_types):
+                dbtype = typeengine.dialect_impl(self.dialect).\
+                                get_dbapi_type(self.dialect.dbapi)
+                if dbtype is not None and \
+                        (not exclude_types or dbtype not in exclude_types):
                     if translate:
                         key = translate.get(key, key)
-                    inputsizes[self.dialect._encoder(key)[0]] = dbtype
+                    if not self.dialect.supports_unicode_binds:
+                        key = self.dialect._encoder(key)[0]
+                    inputsizes[key] = dbtype
             try:
                 self.cursor.setinputsizes(**inputsizes)
             except Exception, e:
-                self.root_connection._handle_dbapi_exception(e, None, None, None, self)
-                raise
+                self.root_connection._handle_dbapi_exception(
+                                e, None, None, None, self)
 
     def _exec_default(self, default, type_):
         if default.is_sequence:
@@ -743,7 +762,7 @@ class DefaultExecutionContext(base.ExecutionContext):
         elif default.is_callable:
             return default.arg(self)
         elif default.is_clause_element:
-            # TODO: expensive branching here should be 
+            # TODO: expensive branching here should be
             # pulled into _exec_scalar()
             conn = self.connection
             c = expression.select([default.arg]).compile(bind=conn)
@@ -809,7 +828,7 @@ class DefaultExecutionContext(base.ExecutionContext):
 
             if self.isinsert:
                 self.inserted_primary_key = [
-                                self.compiled_parameters[0].get(c.key, None) 
+                                self.compiled_parameters[0].get(c.key, None)
                                         for c in self.compiled.\
                                                 statement.table.primary_key
                                 ]
