@@ -28,16 +28,16 @@ __all__ = (
     'AttributeExtension',
     'EXT_CONTINUE',
     'EXT_STOP',
-    'ExtensionOption',
-    'InstrumentationManager',
+    'ONETOMANY',
+    'MANYTOMANY',
+    'MANYTOONE',
+    'NOT_EXTENSION',
     'LoaderStrategy',
     'MapperExtension',
     'MapperOption',
     'MapperProperty',
     'PropComparator',
-    'PropertyOption',
     'SessionExtension',
-    'StrategizedOption',
     'StrategizedProperty',
     )
 
