@@ -1,5 +1,5 @@
 # sql/visitors.py
-# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -24,15 +24,16 @@ http://techspot.zzzeek.org/2008/01/23/expression-transformations/
 """
 
 from collections import deque
-import re
-from sqlalchemy import util
+from .. import util
 import operator
+from .. import exc
 
 __all__ = ['VisitableType', 'Visitable', 'ClauseVisitor',
     'CloningVisitor', 'ReplacingCloningVisitor', 'iterate',
     'iterate_depthfirst', 'traverse_using', 'traverse',
     'cloned_traverse', 'replacement_traverse']
 
+
 class VisitableType(type):
     """Metaclass which assigns a `_compiler_dispatch` method to classes
     having a `__visit_name__` attribute.
@@ -43,7 +44,8 @@ class VisitableType(type):
         def _compiler_dispatch (self, visitor, **kw):
             '''Look for an attribute named "visit_" + self.__visit_name__
             on the visitor, and call it with the same kw params.'''
-            return getattr(visitor, 'visit_%s' % self.__visit_name__)(self, **kw)
+            visit_attr = 'visit_%s' % self.__visit_name__
+            return getattr(visitor, visit_attr)(self, **kw)
 
     Classes having no __visit_name__ attribute will remain unaffected.
     """
@@ -68,21 +70,34 @@ def _generate_dispatch(cls):
             # the string name of the class's __visit_name__ is known at
             # this early stage (import time) so it can be pre-constructed.
             getter = operator.attrgetter("visit_%s" % visit_name)
+
             def _compiler_dispatch(self, visitor, **kw):
-                return getter(visitor)(self, **kw)
+                try:
+                    meth = getter(visitor)
+                except AttributeError:
+                    raise exc.UnsupportedCompilationError(visitor, cls)
+                else:
+                    return meth(self, **kw)
         else:
             # The optimization opportunity is lost for this case because the
             # __visit_name__ is not yet a string. As a result, the visit
             # string has to be recalculated with each compilation.
             def _compiler_dispatch(self, visitor, **kw):
-                return getattr(visitor, 'visit_%s' % self.__visit_name__)(self, **kw)
+                visit_attr = 'visit_%s' % self.__visit_name__
+                try:
+                    meth = getattr(visitor, visit_attr)
+                except AttributeError:
+                    raise exc.UnsupportedCompilationError(visitor, cls)
+                else:
+                    return meth(self, **kw)
 
-        _compiler_dispatch.__doc__  = \
+        _compiler_dispatch.__doc__ = \
           """Look for an attribute named "visit_" + self.__visit_name__
             on the visitor, and call it with the same kw params.
             """
         cls._compiler_dispatch = _compiler_dispatch
 
+
 class Visitable(object):
     """Base class for visitable objects, applies the
     ``VisitableType`` metaclass.
@@ -91,6 +106,7 @@ class Visitable(object):
 
     __metaclass__ = VisitableType
 
+
 class ClauseVisitor(object):
     """Base class for visitor objects which can traverse using
     the traverse() function.
@@ -106,8 +122,10 @@ class ClauseVisitor(object):
                 return meth(obj, **kw)
 
     def iterate(self, obj):
-        """traverse the given expression structure, returning an iterator of all elements."""
+        """traverse the given expression structure, returning an iterator
+        of all elements.
 
+        """
         return iterate(obj, self.__traverse_options__)
 
     def traverse(self, obj):
@@ -143,6 +161,7 @@ class ClauseVisitor(object):
         tail._next = visitor
         return self
 
+
 class CloningVisitor(ClauseVisitor):
     """Base class for visitor objects which can traverse using
     the cloned_traverse() function.
@@ -150,14 +169,18 @@ class CloningVisitor(ClauseVisitor):
     """
 
     def copy_and_process(self, list_):
-        """Apply cloned traversal to the given list of elements, and return the new list."""
+        """Apply cloned traversal to the given list of elements, and return
+        the new list.
 
+        """
         return [self.traverse(x) for x in list_]
 
     def traverse(self, obj):
         """traverse and visit the given expression structure."""
 
-        return cloned_traverse(obj, self.__traverse_options__, self._visitor_dict)
+        return cloned_traverse(
+            obj, self.__traverse_options__, self._visitor_dict)
+
 
 class ReplacingCloningVisitor(CloningVisitor):
     """Base class for visitor objects which can traverse using
@@ -184,6 +207,7 @@ class ReplacingCloningVisitor(CloningVisitor):
                     return e
         return replacement_traverse(obj, self.__traverse_options__, replace)
 
+
 def iterate(obj, opts):
     """traverse the given expression structure, returning an iterator.
 
@@ -197,6 +221,7 @@ def iterate(obj, opts):
         for c in t.get_children(**opts):
             stack.append(c)
 
+
 def iterate_depthfirst(obj, opts):
     """traverse the given expression structure, returning an iterator.
 
@@ -212,25 +237,35 @@ def iterate_depthfirst(obj, opts):
             stack.append(c)
     return iter(traversal)
 
-def traverse_using(iterator, obj, visitors):
-    """visit the given expression structure using the given iterator of objects."""
 
+def traverse_using(iterator, obj, visitors):
+    """visit the given expression structure using the given iterator of
+    objects.
+
+    """
     for target in iterator:
         meth = visitors.get(target.__visit_name__, None)
         if meth:
             meth(target)
     return obj
 
-def traverse(obj, opts, visitors):
-    """traverse and visit the given expression structure using the default iterator."""
 
+def traverse(obj, opts, visitors):
+    """traverse and visit the given expression structure using the default
+     iterator.
+
+    """
     return traverse_using(iterate(obj, opts), obj, visitors)
 
-def traverse_depthfirst(obj, opts, visitors):
-    """traverse and visit the given expression structure using the depth-first iterator."""
 
+def traverse_depthfirst(obj, opts, visitors):
+    """traverse and visit the given expression structure using the
+    depth-first iterator.
+
+    """
     return traverse_using(iterate_depthfirst(obj, opts), obj, visitors)
 
+
 def cloned_traverse(obj, opts, visitors):
     """clone the given expression structure, allowing
     modifications by visitors."""
@@ -242,13 +277,13 @@ def cloned_traverse(obj, opts, visitors):
         if elem in stop_on:
             return elem
         else:
-            if elem not in cloned:
-                cloned[elem] = newelem = elem._clone()
+            if id(elem) not in cloned:
+                cloned[id(elem)] = newelem = elem._clone()
                 newelem._copy_internals(clone=clone)
                 meth = visitors.get(newelem.__visit_name__, None)
                 if meth:
                     meth(newelem)
-            return cloned[elem]
+            return cloned[id(elem)]
 
     if obj is not None:
         obj = clone(obj)
@@ -260,16 +295,16 @@ def replacement_traverse(obj, opts, replace):
     replacement by a given replacement function."""
 
     cloned = util.column_dict()
-    stop_on = util.column_set(opts.get('stop_on', []))
+    stop_on = util.column_set([id(x) for x in opts.get('stop_on', [])])
 
     def clone(elem, **kw):
-        if elem in stop_on or \
+        if id(elem) in stop_on or \
             'no_replacement_traverse' in elem._annotations:
             return elem
         else:
             newelem = replace(elem)
             if newelem is not None:
-                stop_on.add(newelem)
+                stop_on.add(id(newelem))
                 return newelem
             else:
                 if elem not in cloned:
