@@ -1,12 +1,18 @@
-from sqlalchemy.test.testing import eq_, assert_raises
-from sqlalchemy import *
+from sqlalchemy.testing import eq_, assert_raises_message
+from sqlalchemy import exc
 from sqlalchemy.databases import firebird
 from sqlalchemy.exc import ProgrammingError
 from sqlalchemy.sql import table, column
-from sqlalchemy.test import *
+from sqlalchemy import types as sqltypes
+from sqlalchemy.testing import fixtures, AssertsExecutionResults, AssertsCompiledSQL
+from sqlalchemy import testing
+from sqlalchemy.testing import engines
+from sqlalchemy import String, VARCHAR, NVARCHAR, Unicode, Integer,\
+    func, insert, update, MetaData, select, Table, Column, text,\
+    Sequence, Float
+from sqlalchemy import schema
 
-
-class DomainReflectionTest(TestBase, AssertsExecutionResults):
+class DomainReflectionTest(fixtures.TestBase, AssertsExecutionResults):
     "Test Firebird domains"
 
     __only_on__ = 'firebird'
@@ -93,8 +99,8 @@ class DomainReflectionTest(TestBase, AssertsExecutionResults):
         assert isinstance(table.c.dt.type, DateTime)
 
 
-class BuggyDomainReflectionTest(TestBase, AssertsExecutionResults):
-    """Test Firebird domains (and some other reflection bumps), 
+class BuggyDomainReflectionTest(fixtures.TestBase, AssertsExecutionResults):
+    """Test Firebird domains (and some other reflection bumps),
     see [ticket:1663] and http://tracker.firebirdsql.org/browse/CORE-356"""
 
     __only_on__ = 'firebird'
@@ -169,7 +175,7 @@ CREATE DOMAIN DOM_ID INTEGER NOT NULL
 CREATE TABLE A (
 ID DOM_ID /* INTEGER NOT NULL */ DEFAULT 0 )
 """
-    
+
     # the 'default' keyword is lower case here
     TABLE_B = """\
 CREATE TABLE B (
@@ -222,16 +228,16 @@ ID DOM_ID /* INTEGER NOT NULL */ default 0 )
         table_a = Table('a', metadata, autoload=True)
 
         eq_(table_a.c.id.server_default.arg.text, "0")
-    
+
     def test_lowercase_default_name(self):
         metadata = MetaData(testing.db)
 
         table_b = Table('b', metadata, autoload=True)
 
         eq_(table_b.c.id.server_default.arg.text, "0")
-        
 
-class CompileTest(TestBase, AssertsCompiledSQL):
+
+class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
 
     __dialect__ = firebird.FBDialect()
 
@@ -248,6 +254,33 @@ class CompileTest(TestBase, AssertsCompiledSQL):
                             'FROM sometable sometable_1',
                             dialect=dialect)
 
+    def test_varchar_raise(self):
+        for type_ in (
+            String,
+            VARCHAR,
+            String(),
+            VARCHAR(),
+            Unicode,
+            Unicode(),
+        ):
+            type_ = sqltypes.to_instance(type_)
+            assert_raises_message(
+                exc.CompileError,
+                "VARCHAR requires a length on dialect firebird",
+                type_.compile,
+            dialect=firebird.dialect())
+
+            t1 = Table('sometable', MetaData(),
+                Column('somecolumn', type_)
+            )
+            assert_raises_message(
+                exc.CompileError,
+                r"\(in table 'sometable', column 'somecolumn'\)\: "
+                r"(?:N)?VARCHAR requires a length on dialect firebird",
+                schema.CreateTable(t1).compile,
+                dialect=firebird.dialect()
+            )
+
     def test_function(self):
         self.assert_compile(func.foo(1, 2), 'foo(:foo_1, :foo_2)')
         self.assert_compile(func.current_time(), 'CURRENT_TIME')
@@ -319,12 +352,37 @@ class CompileTest(TestBase, AssertsCompiledSQL):
         for type_, args, kw, res in columns:
             self.assert_compile(type_(*args, **kw), res)
 
-class MiscTest(TestBase):
+    def test_quoting_initial_chars(self):
+        self.assert_compile(
+            column("_somecol"),
+            '"_somecol"'
+        )
+        self.assert_compile(
+            column("$somecol"),
+            '"$somecol"'
+        )
+class TypesTest(fixtures.TestBase):
+    __only_on__ = 'firebird'
+
+    @testing.provide_metadata
+    def test_infinite_float(self):
+        metadata = self.metadata
+        t = Table('t', metadata,
+            Column('data', Float)
+        )
+        metadata.create_all()
+        t.insert().execute(data=float('inf'))
+        eq_(t.select().execute().fetchall(),
+            [(float('inf'),)]
+        )
+
+class MiscTest(fixtures.TestBase):
 
     __only_on__ = 'firebird'
 
     @testing.provide_metadata
     def test_strlen(self):
+        metadata = self.metadata
 
         # On FB the length() function is implemented by an external UDF,
         # strlen().  Various SA tests fail because they pass a parameter
@@ -341,13 +399,24 @@ class MiscTest(TestBase):
         select([func.count(t.c.id)], func.length(t.c.name)
                == 5).execute().first()[0] == 1
 
-    def test_server_version_info(self):
-        version = testing.db.dialect.server_version_info
-        assert len(version) == 3, 'Got strange version info: %s' \
-            % repr(version)
+    def test_version_parsing(self):
+        for string, result in [
+            ("WI-V1.5.0.1234 Firebird 1.5", (1, 5, 1234, 'firebird')),
+            ("UI-V6.3.2.18118 Firebird 2.1", (2, 1, 18118, 'firebird')),
+            ("LI-V6.3.3.12981 Firebird 2.0", (2, 0, 12981, 'firebird')),
+            ("WI-V8.1.1.333", (8, 1, 1, 'interbase')),
+            ("WI-V8.1.1.333 Firebird 1.5", (1, 5, 333, 'firebird')),
+        ]:
+            eq_(
+                testing.db.dialect._parse_version_info(string),
+                result
+            )
 
+    # for some reason, this test is hanging on 0.8.  Works
+    # fine on 0.9.  Probably some issue in the testing framework
     @testing.provide_metadata
-    def test_rowcount_flag(self):
+    def _test_rowcount_flag(self):
+        metadata = self.metadata
         engine = engines.testing_engine(options={'enable_rowcount'
                 : True})
         assert engine.dialect.supports_sane_rowcount
@@ -386,3 +455,76 @@ class MiscTest(TestBase):
             (text("select 'hello % world' from rdb$database"),
              'hello % world'):
             eq_(testing.db.scalar(expr), result)
+
+from sqlalchemy.testing.mock import Mock, call
+
+
+class ArgumentTest(fixtures.TestBase):
+    def _dbapi(self):
+        return Mock(
+                    paramstyle='qmark',
+                    connect=Mock(
+                            return_value=Mock(
+                                server_version="UI-V6.3.2.18118 Firebird 2.1",
+                                cursor=Mock(return_value=Mock())
+                            )
+                    )
+                )
+
+    def _engine(self, type_, **kw):
+        dbapi = self._dbapi()
+        kw.update(
+            dict(
+                    module=dbapi,
+                    _initialize=False
+            )
+        )
+        engine = engines.testing_engine("firebird+%s://" % type_,
+                                options=kw)
+        return engine
+
+    def test_retaining_flag_default_kinterbasdb(self):
+        engine = self._engine("kinterbasdb")
+        self._assert_retaining(engine, True)
+
+    def test_retaining_flag_true_kinterbasdb(self):
+        engine = self._engine("kinterbasdb", retaining=True)
+        self._assert_retaining(engine, True)
+
+    def test_retaining_flag_false_kinterbasdb(self):
+        engine = self._engine("kinterbasdb", retaining=False)
+        self._assert_retaining(engine, False)
+
+    def test_retaining_flag_default_fdb(self):
+        engine = self._engine("fdb")
+        self._assert_retaining(engine, True)
+
+    def test_retaining_flag_true_fdb(self):
+        engine = self._engine("fdb", retaining=True)
+        self._assert_retaining(engine, True)
+
+    def test_retaining_flag_false_fdb(self):
+        engine = self._engine("fdb", retaining=False)
+        self._assert_retaining(engine, False)
+
+
+    def _assert_retaining(self, engine, flag):
+        conn = engine.connect()
+        trans = conn.begin()
+        trans.commit()
+        eq_(
+            engine.dialect.dbapi.connect.return_value.commit.mock_calls,
+            [call(flag)]
+        )
+
+        trans = conn.begin()
+        trans.rollback()
+        eq_(
+            engine.dialect.dbapi.connect.return_value.rollback.mock_calls,
+            [call(flag)]
+        )
+
+
+
+
+
