@@ -3,19 +3,27 @@ from sqlalchemy.orm import attributes, instrumentation, exc as orm_exc
 from sqlalchemy.orm.collections import collection
 from sqlalchemy.orm.interfaces import AttributeExtension
 from sqlalchemy import exc as sa_exc
-from test.lib import *
-from test.lib.testing import eq_, ne_, assert_raises, \
+from sqlalchemy.testing import eq_, ne_, assert_raises, \
     assert_raises_message
-from test.lib import fixtures
-from test.lib.util import gc_collect, all_partial_orderings
-from sqlalchemy.util import cmp, jython, topological
+from sqlalchemy.testing import fixtures
+from sqlalchemy.testing.util import gc_collect, all_partial_orderings
+from sqlalchemy.util import jython
 from sqlalchemy import event
+from sqlalchemy import testing
+from sqlalchemy.testing.mock import Mock, call
+from sqlalchemy.orm.state import InstanceState
 
 # global for pickling tests
 MyTest = None
 MyTest2 = None
 
 
+
+def _set_callable(state, dict_, key, callable_):
+    fn = InstanceState._row_processor(state.manager, callable_, key)
+    fn(state, dict_, None)
+
+
 class AttributeImplAPITest(fixtures.MappedTest):
     def _scalar_obj_fixture(self):
         class A(object):
@@ -188,7 +196,7 @@ class AttributesTest(fixtures.ORMTest):
         u.email_address = 'lala@123.com'
         self.assert_(u.user_id == 7 and u.user_name == 'john'
                      and u.email_address == 'lala@123.com')
-        attributes.instance_state(u).commit_all(attributes.instance_dict(u))
+        attributes.instance_state(u)._commit_all(attributes.instance_dict(u))
         self.assert_(u.user_id == 7 and u.user_name == 'john'
                      and u.email_address == 'lala@123.com')
         u.user_name = 'heythere'
@@ -205,9 +213,6 @@ class AttributesTest(fixtures.ORMTest):
                 uselist=False, useobject=False)
         attributes.register_attribute(MyTest, 'email_address',
                 uselist=False, useobject=False)
-        attributes.register_attribute(MyTest, 'some_mutable_data',
-                mutable_scalars=True, copy_function=list,
-                compare_function=cmp, uselist=False, useobject=False)
         attributes.register_attribute(MyTest2, 'a', uselist=False,
                 useobject=False)
         attributes.register_attribute(MyTest2, 'b', uselist=False,
@@ -225,7 +230,6 @@ class AttributesTest(fixtures.ORMTest):
         o = MyTest()
         o.mt2.append(MyTest2())
         o.user_id=7
-        o.some_mutable_data = [1,2,3]
         o.mt2[0].a = 'abcde'
         pk_o = pickle.dumps(o)
 
@@ -268,11 +272,11 @@ class AttributesTest(fixtures.ORMTest):
         self.assert_(o4.user_id == 7)
         self.assert_(o4.user_name is None)
         self.assert_(o4.email_address is None)
-        self.assert_(o4.some_mutable_data == [1,2,3])
         self.assert_(len(o4.mt2) == 1)
         self.assert_(o4.mt2[0].a == 'abcde')
         self.assert_(o4.mt2[0].b is None)
 
+    @testing.requires.predictable_gc
     def test_state_gc(self):
         """test that InstanceState always has a dict, even after host
         object gc'ed."""
@@ -284,7 +288,7 @@ class AttributesTest(fixtures.ORMTest):
         f = Foo()
         state = attributes.instance_state(f)
         f.bar = "foo"
-        assert state.dict == {'bar':'foo', state.manager.STATE_ATTR:state}
+        eq_(state.dict, {'bar': 'foo', state.manager.STATE_ATTR: state})
         del f
         gc_collect()
         assert state.obj() is None
@@ -328,28 +332,28 @@ class AttributesTest(fixtures.ORMTest):
         attributes.register_attribute(Foo, 'b', uselist=False, useobject=False)
 
         f = Foo()
-        attributes.instance_state(f).expire(attributes.instance_dict(f),
+        attributes.instance_state(f)._expire(attributes.instance_dict(f),
                 set())
         eq_(f.a, 'this is a')
         eq_(f.b, 12)
         f.a = 'this is some new a'
-        attributes.instance_state(f).expire(attributes.instance_dict(f),
+        attributes.instance_state(f)._expire(attributes.instance_dict(f),
                 set())
         eq_(f.a, 'this is a')
         eq_(f.b, 12)
-        attributes.instance_state(f).expire(attributes.instance_dict(f),
+        attributes.instance_state(f)._expire(attributes.instance_dict(f),
                 set())
         f.a = 'this is another new a'
         eq_(f.a, 'this is another new a')
         eq_(f.b, 12)
-        attributes.instance_state(f).expire(attributes.instance_dict(f),
+        attributes.instance_state(f)._expire(attributes.instance_dict(f),
                 set())
         eq_(f.a, 'this is a')
         eq_(f.b, 12)
         del f.a
         eq_(f.a, None)
         eq_(f.b, 12)
-        attributes.instance_state(f).commit_all(attributes.instance_dict(f),
+        attributes.instance_state(f)._commit_all(attributes.instance_dict(f),
                 set())
         eq_(f.a, None)
         eq_(f.b, 12)
@@ -368,7 +372,7 @@ class AttributesTest(fixtures.ORMTest):
         attributes.register_attribute(MyTest, 'b', uselist=False, useobject=False)
 
         m = MyTest()
-        attributes.instance_state(m).expire(attributes.instance_dict(m), set())
+        attributes.instance_state(m)._expire(attributes.instance_dict(m), set())
         assert 'a' not in m.__dict__
         m2 = pickle.loads(pickle.dumps(m))
         assert 'a' not in m2.__dict__
@@ -404,7 +408,7 @@ class AttributesTest(fixtures.ORMTest):
         self.assert_(u.user_id == 7 and u.user_name == 'john'
                      and u.addresses[0].email_address == 'lala@123.com')
         (u,
-         attributes.instance_state(a).commit_all(attributes.instance_dict(a)))
+         attributes.instance_state(a)._commit_all(attributes.instance_dict(a)))
         self.assert_(u.user_id == 7 and u.user_name == 'john'
                      and u.addresses[0].email_address == 'lala@123.com')
 
@@ -438,17 +442,17 @@ class AttributesTest(fixtures.ORMTest):
 
             def append(self, state, child, initiator):
                 if commit:
-                    state.commit_all(state.dict)
+                    state._commit_all(state.dict)
                 return child
 
             def remove(self, state, child, initiator):
                 if commit:
-                    state.commit_all(state.dict)
+                    state._commit_all(state.dict)
                 return child
 
             def set(self, state, child, oldchild, initiator):
                 if commit:
-                    state.commit_all(state.dict)
+                    state._commit_all(state.dict)
                 return child
 
         instrumentation.register_class(Foo)
@@ -485,7 +489,7 @@ class AttributesTest(fixtures.ORMTest):
 
         def create_hist():
             def hist(key, shouldmatch, fn, *arg):
-                attributes.instance_state(f1).commit_all(attributes.instance_dict(f1))
+                attributes.instance_state(f1)._commit_all(attributes.instance_dict(f1))
                 fn(*arg)
                 histories.append((shouldmatch,
                                  attributes.get_history(f1, key)))
@@ -557,15 +561,14 @@ class AttributesTest(fixtures.ORMTest):
         x.bars
         b = Bar(id=4)
         b.foos.append(x)
-        attributes.instance_state(x).expire_attributes(attributes.instance_dict(x),
+        attributes.instance_state(x)._expire_attributes(attributes.instance_dict(x),
                 ['bars'])
         assert_raises(AssertionError, b.foos.remove, x)
 
 
     def test_scalar_listener(self):
 
-        # listeners on ScalarAttributeImpl and
-        # MutableScalarAttributeImpl aren't used normally. test that
+        # listeners on ScalarAttributeImpl aren't used normally. test that
         # they work for the benefit of user extensions
 
         class Foo(object):
@@ -586,27 +589,18 @@ class AttributesTest(fixtures.ORMTest):
 
         instrumentation.register_class(Foo)
         attributes.register_attribute(Foo, 'x', uselist=False,
-                mutable_scalars=False, useobject=False,
+                useobject=False,
                 extension=ReceiveEvents())
-        attributes.register_attribute(Foo, 'y', uselist=False,
-                mutable_scalars=True, useobject=False,
-                copy_function=lambda x: x, extension=ReceiveEvents())
 
         f = Foo()
         f.x = 5
         f.x = 17
         del f.x
-        f.y = [1,2,3]
-        f.y = [4,5,6]
-        del f.y
 
         eq_(results, [
             ('set', f, 5, attributes.NEVER_SET),
             ('set', f, 17, 5),
             ('remove', f, 17),
-            ('set', f, [1,2,3], attributes.NEVER_SET),
-            ('set', f, [4,5,6], [1,2,3]),
-            ('remove', f, [4,5,6])
         ])
 
     def test_lazytrackparent(self):
@@ -615,8 +609,10 @@ class AttributesTest(fixtures.ORMTest):
 
         """
 
-        class Post(object):pass
-        class Blog(object):pass
+        class Post(object):
+            pass
+        class Blog(object):
+            pass
         instrumentation.register_class(Post)
         instrumentation.register_class(Blog)
 
@@ -631,11 +627,11 @@ class AttributesTest(fixtures.ORMTest):
         # create objects as if they'd been freshly loaded from the database (without history)
         b = Blog()
         p1 = Post()
-        attributes.instance_state(b).set_callable(attributes.instance_dict(b),
-                                                    'posts', lambda passive:[p1])
-        attributes.instance_state(p1).set_callable(attributes.instance_dict(p1),
-                                                    'blog', lambda passive:b)
-        p1, attributes.instance_state(b).commit_all(attributes.instance_dict(b))
+        _set_callable(attributes.instance_state(b), attributes.instance_dict(b),
+                                                    'posts', lambda state, passive:[p1])
+        _set_callable(attributes.instance_state(p1), attributes.instance_dict(p1),
+                                                    'blog', lambda state, passive:b)
+        p1, attributes.instance_state(b)._commit_all(attributes.instance_dict(b))
 
         # no orphans (called before the lazy loaders fire off)
         assert attributes.has_parent(Blog, p1, 'posts', optimistic=True)
@@ -743,7 +739,7 @@ class AttributesTest(fixtures.ORMTest):
         x.element = el
         eq_(attributes.get_state_history(attributes.instance_state(x),
             'element'), ([el], (), ()))
-        attributes.instance_state(x).commit_all(attributes.instance_dict(x))
+        attributes.instance_state(x)._commit_all(attributes.instance_dict(x))
         added, unchanged, deleted = \
             attributes.get_state_history(attributes.instance_state(x),
                 'element')
@@ -776,7 +772,7 @@ class AttributesTest(fixtures.ORMTest):
         attributes.register_attribute(Bar, 'id', uselist=False,
                 useobject=True)
         x = Foo()
-        attributes.instance_state(x).commit_all(attributes.instance_dict(x))
+        attributes.instance_state(x)._commit_all(attributes.instance_dict(x))
         x.col2.append(bar4)
         eq_(attributes.get_state_history(attributes.instance_state(x),
             'col2'), ([bar4], [bar1, bar2, bar3], []))
@@ -817,31 +813,6 @@ class AttributesTest(fixtures.ORMTest):
         b3.element = f4
         assert attributes.has_parent(Bar, f4, 'element')
 
-    def test_mutablescalars(self):
-        """test detection of changes on mutable scalar items"""
-
-        class Foo(object):
-            pass
-
-        instrumentation.register_class(Foo)
-        attributes.register_attribute(Foo, 'element', uselist=False,
-                copy_function=lambda x: [y for y in x],
-                mutable_scalars=True, useobject=False)
-        x = Foo()
-        x.element = ['one', 'two', 'three']
-        attributes.instance_state(x).commit_all(attributes.instance_dict(x))
-        x.element[1] = 'five'
-        assert attributes.instance_state(x).modified
-        instrumentation.unregister_class(Foo)
-        instrumentation.register_class(Foo)
-        attributes.register_attribute(Foo, 'element', uselist=False,
-                useobject=False)
-        x = Foo()
-        x.element = ['one', 'two', 'three']
-        attributes.instance_state(x).commit_all(attributes.instance_dict(x))
-        x.element[1] = 'five'
-        assert not attributes.instance_state(x).modified
-
     def test_descriptorattributes(self):
         """changeset: 1633 broke ability to use ORM to map classes with
         unusual descriptor attributes (for example, classes that inherit
@@ -990,14 +961,6 @@ class GetNoValueTest(fixtures.ORMTest):
         )
         assert 'attr' not in dict_
 
-    def test_passive_no_result_empty(self):
-        attr, state, dict_ = self._fixture(None)
-        eq_(
-            attr.get(state, dict_, passive=attributes.PASSIVE_NO_RESULT),
-            None
-        )
-        assert 'attr' in dict_
-
     def test_off_empty(self):
         attr, state, dict_ = self._fixture(None)
         eq_(
@@ -1217,7 +1180,10 @@ class CyclicBackrefAssertionTest(fixtures.TestBase):
         b1 = B()
         assert_raises_message(
             ValueError,
-            "Object <B at .*> not associated with attribute of type C.a",
+            'Bidirectional attribute conflict detected: '
+            'Passing object <B at .*> to attribute "C.a" '
+            'triggers a modify event on attribute "C.b" '
+            'via the backref "B.c".',
             setattr, c1, 'a', b1
         )
 
@@ -1227,10 +1193,14 @@ class CyclicBackrefAssertionTest(fixtures.TestBase):
         b1 = B()
         assert_raises_message(
             ValueError,
-            "Object <B at .*> not associated with attribute of type C.a",
+            'Bidirectional attribute conflict detected: '
+            'Passing object <B at .*> to attribute "C.a" '
+            'triggers a modify event on attribute "C.b" '
+            'via the backref "B.c".',
             c1.a.append, b1
         )
 
+
     def _scalar_fixture(self):
         class A(object):
             pass
@@ -1272,10 +1242,38 @@ class CyclicBackrefAssertionTest(fixtures.TestBase):
 
         return A, B, C
 
-class PendingBackrefTest(fixtures.ORMTest):
-    def setup(self):
-        global Post, Blog, called, lazy_load
+    def _broken_collection_fixture(self):
+        class A(object):
+            pass
+        class B(object):
+            pass
+        instrumentation.register_class(A)
+        instrumentation.register_class(B)
 
+        attributes.register_attribute(A, 'b', backref='a1', useobject=True)
+        attributes.register_attribute(B, 'a1', backref='b', useobject=True,
+                                                uselist=True)
+
+        attributes.register_attribute(B, 'a2', backref='b', useobject=True,
+                                                uselist=True)
+
+        return A, B
+
+    def test_broken_collection_assertion(self):
+        A, B = self._broken_collection_fixture()
+        b1 = B()
+        a1 = A()
+        assert_raises_message(
+            ValueError,
+            'Bidirectional attribute conflict detected: '
+            'Passing object <A at .*> to attribute "B.a2" '
+            'triggers a modify event on attribute "B.a1" '
+            'via the backref "A.b".',
+            b1.a2.append, a1
+        )
+
+class PendingBackrefTest(fixtures.ORMTest):
+    def _fixture(self):
         class Post(object):
             def __init__(self, name):
                 self.name = name
@@ -1290,15 +1288,7 @@ class PendingBackrefTest(fixtures.ORMTest):
             def __eq__(self, other):
                 return other is not None and other.name == self.name
 
-        called = [0]
-
-        lazy_load = []
-        def lazy_posts(state, passive):
-            if passive is not attributes.PASSIVE_NO_FETCH:
-                called[0] += 1
-                return lazy_load
-            else:
-                return attributes.PASSIVE_NO_RESULT
+        lazy_posts = Mock()
 
         instrumentation.register_class(Post)
         instrumentation.register_class(Blog)
@@ -1308,30 +1298,54 @@ class PendingBackrefTest(fixtures.ORMTest):
                 backref='blog', callable_=lazy_posts, trackparent=True,
                 useobject=True)
 
+        return Post, Blog, lazy_posts
+
     def test_lazy_add(self):
-        global lazy_load
+        Post, Blog, lazy_posts = self._fixture()
 
         p1, p2, p3 = Post("post 1"), Post("post 2"), Post("post 3")
-        lazy_load = [p1, p2, p3]
+        lazy_posts.return_value = attributes.PASSIVE_NO_RESULT
 
         b = Blog("blog 1")
+        b1_state = attributes.instance_state(b)
+
         p = Post("post 4")
 
         p.blog = b
+        eq_(
+            lazy_posts.mock_calls, [
+                call(b1_state, attributes.PASSIVE_NO_FETCH)
+            ]
+        )
+
         p = Post("post 5")
+
+        # setting blog doesnt call 'posts' callable, calls with no fetch
         p.blog = b
-        # setting blog doesnt call 'posts' callable
-        assert called[0] == 0
+        eq_(
+            lazy_posts.mock_calls, [
+                call(b1_state, attributes.PASSIVE_NO_FETCH),
+                call(b1_state, attributes.PASSIVE_NO_FETCH)
+            ]
+        )
+
+        lazy_posts.return_value = [p1, p2, p3]
 
         # calling backref calls the callable, populates extra posts
-        assert b.posts == [p1, p2, p3, Post("post 4"), Post("post 5")]
-        assert called[0] == 1
+        eq_(b.posts, [p1, p2, p3, Post("post 4"), Post("post 5")])
+        eq_(
+            lazy_posts.mock_calls, [
+                call(b1_state, attributes.PASSIVE_NO_FETCH),
+                call(b1_state, attributes.PASSIVE_NO_FETCH),
+                call(b1_state, attributes.PASSIVE_OFF)
+            ]
+        )
 
     def test_lazy_history(self):
-        global lazy_load
+        Post, Blog, lazy_posts = self._fixture()
 
         p1, p2, p3 = Post("post 1"), Post("post 2"), Post("post 3")
-        lazy_load = [p1, p2, p3]
+        lazy_posts.return_value = [p1, p2, p3]
 
         b = Blog("blog 1")
         p = Post("post 4")
@@ -1339,67 +1353,136 @@ class PendingBackrefTest(fixtures.ORMTest):
 
         p4 = Post("post 5")
         p4.blog = b
-        assert called[0] == 0
+
+        eq_(lazy_posts.call_count, 1)
+
         eq_(attributes.instance_state(b).
                 get_history('posts', attributes.PASSIVE_OFF),
                             ([p, p4], [p1, p2, p3], []))
-        assert called[0] == 1
+        eq_(lazy_posts.call_count, 1)
+
+    def test_passive_history_collection_never_set(self):
+        Post, Blog, lazy_posts = self._fixture()
+
+        lazy_posts.return_value = attributes.PASSIVE_NO_RESULT
 
-    def test_state_on_add_remove(self):
         b = Blog("blog 1")
         p = Post("post 1")
-        p.blog = b
-        p.blog = None
 
-        eq_(called[0], 0)
+        state, dict_ = attributes.instance_state(b), attributes.instance_dict(b)
+
+        # this sets up NEVER_SET on b.posts
+        p.blog = b
+
+        eq_(state.committed_state, {"posts": attributes.NEVER_SET})
+        assert 'posts' not in dict_
+
+        # then suppose the object was made transient again,
+        # the lazy loader would return this
+        lazy_posts.return_value = attributes.ATTR_EMPTY
+
+        p2 = Post('asdf')
+        p2.blog = b
+
+        eq_(state.committed_state, {"posts": attributes.NEVER_SET})
+        eq_(dict_['posts'], [p2])
+
+        # then this would fail.
+        eq_(
+            Blog.posts.impl.get_history(state, dict_, passive=True),
+            ([p2], (), ())
+        )
+
+        eq_(
+            Blog.posts.impl.get_all_pending(state, dict_),
+            [(attributes.instance_state(p2), p2)]
+        )
+
+    def test_state_on_add_remove(self):
+        Post, Blog, lazy_posts = self._fixture()
+        lazy_posts.return_value = attributes.PASSIVE_NO_RESULT
+
+        b = Blog("blog 1")
+        b1_state = attributes.instance_state(b)
+        p = Post("post 1")
+        p.blog = b
+        eq_(lazy_posts.mock_calls,
+                [call(b1_state, attributes.PASSIVE_NO_FETCH)])
+        p.blog = None
+        eq_(lazy_posts.mock_calls,
+                [call(b1_state, attributes.PASSIVE_NO_FETCH),
+                call(b1_state, attributes.PASSIVE_NO_FETCH)])
+        lazy_posts.return_value = []
         eq_(b.posts, [])
-        eq_(called[0], 1)
+        eq_(lazy_posts.mock_calls,
+                [call(b1_state, attributes.PASSIVE_NO_FETCH),
+                call(b1_state, attributes.PASSIVE_NO_FETCH),
+                call(b1_state, attributes.PASSIVE_OFF)])
 
     def test_pending_combines_with_lazy(self):
-        global lazy_load
+        Post, Blog, lazy_posts = self._fixture()
+
+        lazy_posts.return_value = attributes.PASSIVE_NO_RESULT
+
         b = Blog("blog 1")
         p = Post("post 1")
         p2 = Post("post 2")
         p.blog = b
-        lazy_load = [p, p2]
+        eq_(lazy_posts.call_count, 1)
+
+        lazy_posts.return_value = [p, p2]
+
         # lazy loaded + pending get added together.
         # This isn't seen often with the ORM due
         # to usual practices surrounding the
         # load/flush/load cycle.
         eq_(b.posts, [p, p2, p])
-        eq_(called[0], 1)
+        eq_(lazy_posts.call_count, 2)
 
     def test_normal_load(self):
-        global lazy_load
-        lazy_load = (p1, p2, p3) = [Post("post 1"), Post("post 2"), Post("post 3")]
-        called[0] = 0
+        Post, Blog, lazy_posts = self._fixture()
+
+        lazy_posts.return_value = \
+            (p1, p2, p3) = [Post("post 1"), Post("post 2"), Post("post 3")]
 
         b = Blog("blog 1")
 
         # assign without using backref system
         p2.__dict__['blog'] = b
 
-        assert b.posts == [Post("post 1"), Post("post 2"), Post("post 3")]
-        assert called[0] == 1
+        eq_(b.posts, [Post("post 1"), Post("post 2"), Post("post 3")])
+
+        eq_(lazy_posts.call_count, 1)
         p2.blog = None
         p4 = Post("post 4")
         p4.blog = b
-        assert b.posts == [Post("post 1"), Post("post 3"), Post("post 4")]
-        assert called[0] == 1
+        eq_(b.posts, [Post("post 1"), Post("post 3"), Post("post 4")])
+
+        b_state = attributes.instance_state(b)
+
+        eq_(lazy_posts.call_count, 1)
+        eq_(lazy_posts.mock_calls,
+            [call(b_state, attributes.PASSIVE_OFF)])
 
-        called[0] = 0
-        lazy_load = (p1, p2, p3) = [Post("post 1"), Post("post 2"), Post("post 3")]
 
     def test_commit_removes_pending(self):
-        global lazy_load
-        lazy_load = (p1, ) = [Post("post 1"), ]
-        called[0] = 0
+        Post, Blog, lazy_posts = self._fixture()
 
+        p1 = Post("post 1")
+
+        lazy_posts.return_value = attributes.PASSIVE_NO_RESULT
         b = Blog("blog 1")
         p1.blog = b
-        attributes.instance_state(b).commit_all(attributes.instance_dict(b))
-        attributes.instance_state(p1).commit_all(attributes.instance_dict(p1))
-        assert b.posts == [Post("post 1")]
+
+        b_state = attributes.instance_state(b)
+        p1_state = attributes.instance_state(p1)
+        b_state._commit_all(attributes.instance_dict(b))
+        p1_state._commit_all(attributes.instance_dict(p1))
+        lazy_posts.return_value = [p1]
+        eq_(b.posts, [Post("post 1")])
+        eq_(lazy_posts.mock_calls,
+            [call(b_state, attributes.PASSIVE_NO_FETCH),
+            call(b_state, attributes.PASSIVE_OFF)])
 
 class HistoryTest(fixtures.TestBase):
 
@@ -1435,7 +1518,7 @@ class HistoryTest(fixtures.TestBase):
                     'someattr', **kw)
 
     def _commit_someattr(self, f):
-        attributes.instance_state(f).commit(attributes.instance_dict(f),
+        attributes.instance_state(f)._commit(attributes.instance_dict(f),
                 ['someattr'])
 
     def _someattr_committed_state(self, f):
@@ -1604,7 +1687,19 @@ class HistoryTest(fixtures.TestBase):
         Foo = self._fixture(uselist=True, useobject=True,
                                 active_history=True)
         f = Foo()
-        eq_(self._someattr_history(f, passive=True), ((), [], ()))
+        eq_(self._someattr_history(f, passive=True), ((), (), ()))
+
+    def test_scalar_obj_never_set(self):
+        Foo = self._fixture(uselist=False, useobject=True,
+                                active_history=True)
+        f = Foo()
+        eq_(self._someattr_history(f, passive=True), ((), (), ()))
+
+    def test_scalar_never_set(self):
+        Foo = self._fixture(uselist=False, useobject=False,
+                                active_history=True)
+        f = Foo()
+        eq_(self._someattr_history(f, passive=True), ((), (), ()))
 
     def test_scalar_active_set(self):
         Foo = self._fixture(uselist=False, useobject=False,
@@ -1700,91 +1795,6 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), (['two'], (), ()))
 
 
-    def test_mutable_scalar_init(self):
-        Foo = self._fixture(uselist=False, useobject=False,
-                                active_history=False,
-                                mutable_scalars=True,copy_function=dict)
-        f = Foo()
-        eq_(self._someattr_history(f), ((), (), ()))
-
-    def test_mutable_scalar_no_init_side_effect(self):
-        Foo = self._fixture(uselist=False, useobject=False,
-                                active_history=False,
-                                mutable_scalars=True,copy_function=dict)
-        f = Foo()
-        self._someattr_history(f)
-        assert 'someattr' not in f.__dict__
-        assert 'someattr' not in attributes.instance_state(f).committed_state
-
-    def test_mutable_scalar_set(self):
-        Foo = self._fixture(uselist=False, useobject=False,
-                                active_history=False,
-                                mutable_scalars=True,copy_function=dict)
-        f = Foo()
-        f.someattr = {'foo': 'hi'}
-        eq_(self._someattr_history(f), ([{'foo': 'hi'}], (), ()))
-
-    def test_mutable_scalar_set_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False,
-                                active_history=False,
-                                mutable_scalars=True,copy_function=dict)
-        f = Foo()
-        f.someattr = {'foo': 'hi'}
-        self._commit_someattr(f)
-        eq_(self._someattr_history(f), ((), [{'foo': 'hi'}], ()))
-        eq_(attributes.instance_state(f).committed_state['someattr'],
-            {'foo': 'hi'})
-
-    def test_mutable_scalar_set_commit_reset(self):
-        Foo = self._fixture(uselist=False, useobject=False,
-                                active_history=False,
-                                mutable_scalars=True,copy_function=dict)
-        f = Foo()
-        f.someattr = {'foo': 'hi'}
-        self._commit_someattr(f)
-        f.someattr['foo'] = 'there'
-        eq_(self._someattr_history(f), ([{'foo': 'there'}], (), [{'foo': 'hi'}]))
-        eq_(attributes.get_state_history(attributes.instance_state(f),
-            'someattr'), ([{'foo': 'there'}], (), [{'foo': 'hi'}]))
-
-    def test_mutable_scalar_set_commit_reset_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False,
-                                active_history=False,
-                                mutable_scalars=True,copy_function=dict)
-        f = Foo()
-        f.someattr = {'foo': 'hi'}
-        self._commit_someattr(f)
-        f.someattr['foo'] = 'there'
-        self._commit_someattr(f)
-        eq_(self._someattr_history(f), ((), [{'foo': 'there'}], ()))
-
-    def test_mutable_scalar_set_dict(self):
-        Foo = self._fixture(uselist=False, useobject=False,
-                                active_history=False,
-                                mutable_scalars=True,copy_function=dict)
-        f = Foo()
-        f.__dict__['someattr'] = {'foo': 'new'}
-        eq_(self._someattr_history(f), ((), [{'foo': 'new'}], ()))
-
-    def test_mutable_scalar_set_dict_set(self):
-        Foo = self._fixture(uselist=False, useobject=False,
-                                active_history=False,
-                                mutable_scalars=True,copy_function=dict)
-        f = Foo()
-        f.__dict__['someattr'] = {'foo': 'new'}
-        eq_(self._someattr_history(f), ((), [{'foo': 'new'}], ()))
-        f.someattr = {'foo': 'old'}
-        eq_(self._someattr_history(f), ([{'foo': 'old'}], (), [{'foo': 'new'}]))
-
-    def test_mutable_scalar_set_dict_set_commit(self):
-        Foo = self._fixture(uselist=False, useobject=False,
-                                active_history=False,
-                                mutable_scalars=True,copy_function=dict)
-        f = Foo()
-        f.__dict__['someattr'] = {'foo': 'new'}
-        f.someattr = {'foo': 'old'}
-        self._commit_someattr(f)
-        eq_(self._someattr_history(f), ((), [{'foo': 'old'}], ()))
 
     def test_scalar_inplace_mutation_set(self):
         Foo = self._fixture(uselist=False, useobject=False,
@@ -1840,6 +1850,23 @@ class HistoryTest(fixtures.TestBase):
         f.someattr = ['a']
         eq_(self._someattr_history(f), ([['a']], (), ()))
 
+
+    def test_scalar_inplace_mutation_replace_self_flag_modified_set(self):
+        Foo = self._fixture(uselist=False, useobject=False,
+                                active_history=False)
+        f = Foo()
+        f.someattr = {'a': 'b'}
+        self._commit_someattr(f)
+        eq_(self._someattr_history(f), ((), [{'a': 'b'}], ()))
+
+        # set the attribute to itself; this places a copy
+        # in committed_state
+        f.someattr = f.someattr
+
+        attributes.flag_modified(f, 'someattr')
+        eq_(self._someattr_history(f), ([{'a': 'b'}], (), ()))
+
+
     def test_use_object_init(self):
         Foo, Bar = self._two_obj_fixture(uselist=False)
         f = Foo()
@@ -1948,6 +1975,7 @@ class HistoryTest(fixtures.TestBase):
         f.someattr = there
         eq_(self._someattr_history(f), ([there], (), ()))
 
+
     def test_object_collections_set(self):
         # TODO: break into individual tests
 
@@ -1987,7 +2015,7 @@ class HistoryTest(fixtures.TestBase):
         f = Foo()
         collection = attributes.init_collection(f, 'someattr')
         collection.append_without_event(new)
-        attributes.instance_state(f).commit_all(attributes.instance_dict(f))
+        attributes.instance_state(f)._commit_all(attributes.instance_dict(f))
         eq_(attributes.get_state_history(attributes.instance_state(f),
             'someattr'), ((), [new], ()))
         f.someattr = [old]
@@ -2073,7 +2101,7 @@ class HistoryTest(fixtures.TestBase):
         f.someattr.append(new)
         eq_(attributes.get_state_history(attributes.instance_state(f),
             'someattr'), ([old, new], [hi], [there]))
-        attributes.instance_state(f).commit(attributes.instance_dict(f),
+        attributes.instance_state(f)._commit(attributes.instance_dict(f),
                 ['someattr'])
         eq_(attributes.get_state_history(attributes.instance_state(f),
             'someattr'), ((), [hi, old, new], ()))
@@ -2088,20 +2116,20 @@ class HistoryTest(fixtures.TestBase):
         f.__dict__['id'] = 1
         collection = attributes.init_collection(f, 'someattr')
         collection.append_without_event(new)
-        attributes.instance_state(f).commit_all(attributes.instance_dict(f))
+        attributes.instance_state(f)._commit_all(attributes.instance_dict(f))
         eq_(attributes.get_state_history(attributes.instance_state(f),
             'someattr'), ((), [new], ()))
         f.someattr.append(old)
         eq_(attributes.get_state_history(attributes.instance_state(f),
             'someattr'), ([old], [new], []))
-        attributes.instance_state(f).commit(attributes.instance_dict(f),
+        attributes.instance_state(f)._commit(attributes.instance_dict(f),
                 ['someattr'])
         eq_(attributes.get_state_history(attributes.instance_state(f),
             'someattr'), ((), [new, old], ()))
         f = Foo()
         collection = attributes.init_collection(f, 'someattr')
         collection.append_without_event(new)
-        attributes.instance_state(f).commit_all(attributes.instance_dict(f))
+        attributes.instance_state(f)._commit_all(attributes.instance_dict(f))
         eq_(attributes.get_state_history(attributes.instance_state(f),
             'someattr'), ((), [new], ()))
         f.id = 1
@@ -2130,7 +2158,7 @@ class HistoryTest(fixtures.TestBase):
         f.someattr.append(hi)
         eq_(attributes.get_state_history(attributes.instance_state(f),
             'someattr'), ([hi, there, hi], [], []))
-        attributes.instance_state(f).commit_all(attributes.instance_dict(f))
+        attributes.instance_state(f)._commit_all(attributes.instance_dict(f))
         eq_(attributes.get_state_history(attributes.instance_state(f),
             'someattr'), ((), [hi, there, hi], ()))
         f.someattr = []
@@ -2225,7 +2253,7 @@ class LazyloadHistoryTest(fixtures.TestBase):
         eq_(attributes.get_state_history(attributes.instance_state(f),
             'bars'), ([bar4], [], []))
         lazy_load = [bar1, bar2, bar3]
-        attributes.instance_state(f).expire_attributes(attributes.instance_dict(f),
+        attributes.instance_state(f)._expire_attributes(attributes.instance_dict(f),
                 ['bars'])
         eq_(attributes.get_state_history(attributes.instance_state(f),
             'bars'), ((), [bar1, bar2, bar3], ()))
@@ -2447,6 +2475,46 @@ class ListenerTest(fixtures.ORMTest):
         f1.barset.add(b1)
         assert f1.barset.pop().data == 'some bar appended'
 
+    def test_none_on_collection_event(self):
+        """test that append/remove of None in collections emits events.
+
+        This is new behavior as of 0.8.
+
+        """
+        class Foo(object):
+            pass
+        class Bar(object):
+            pass
+        instrumentation.register_class(Foo)
+        instrumentation.register_class(Bar)
+        attributes.register_attribute(Foo, 'barlist', uselist=True,
+                useobject=True)
+        canary = []
+        def append(state, child, initiator):
+            canary.append((state, child))
+        def remove(state, child, initiator):
+            canary.append((state, child))
+        event.listen(Foo.barlist, 'append', append)
+        event.listen(Foo.barlist, 'remove', remove)
+
+        b1, b2 = Bar(), Bar()
+        f1 = Foo()
+        f1.barlist.append(None)
+        eq_(canary, [(f1, None)])
+
+        canary[:] = []
+        f1 = Foo()
+        f1.barlist = [None, b2]
+        eq_(canary, [(f1, None), (f1, b2)])
+
+        canary[:] = []
+        f1 = Foo()
+        f1.barlist = [b1, None, b2]
+        eq_(canary, [(f1, b1), (f1, None), (f1, b2)])
+
+        f1.barlist.remove(None)
+        eq_(canary, [(f1, b1), (f1, None), (f1, b2), (f1, None)])
+
     def test_propagate(self):
         classes = [None, None, None]
         canary = []
@@ -2531,3 +2599,63 @@ class ListenerTest(fixtures.ORMTest):
 
             teardown()
 
+
+class TestUnlink(fixtures.TestBase):
+    def setUp(self):
+        class A(object):
+            pass
+        class B(object):
+            pass
+        self.A = A
+        self.B = B
+        instrumentation.register_class(A)
+        instrumentation.register_class(B)
+        attributes.register_attribute(A, 'bs', uselist=True,
+                useobject=True)
+
+    def test_expired(self):
+        A, B = self.A, self.B
+        a1 = A()
+        coll = a1.bs
+        a1.bs.append(B())
+        state = attributes.instance_state(a1)
+        state._expire(state.dict, set())
+        assert_raises(
+            Warning,
+            coll.append, B()
+        )
+
+    def test_replaced(self):
+        A, B = self.A, self.B
+        a1 = A()
+        coll = a1.bs
+        a1.bs.append(B())
+        a1.bs = []
+        # a bulk replace empties the old collection
+        assert len(coll) == 0
+        coll.append(B())
+        assert len(coll) == 1
+
+    def test_pop_existing(self):
+        A, B = self.A, self.B
+        a1 = A()
+        coll = a1.bs
+        a1.bs.append(B())
+        state = attributes.instance_state(a1)
+        state._reset(state.dict, "bs")
+        assert_raises(
+            Warning,
+            coll.append, B()
+        )
+
+    def test_ad_hoc_lazy(self):
+        A, B = self.A, self.B
+        a1 = A()
+        coll = a1.bs
+        a1.bs.append(B())
+        state = attributes.instance_state(a1)
+        _set_callable(state, state.dict, "bs", lambda: B())
+        assert_raises(
+            Warning,
+            coll.append, B()
+        )
