@@ -1,17 +1,17 @@
-from test.lib.testing import assert_raises, assert_raises_message
+from sqlalchemy.testing import assert_raises, assert_raises_message
 import sqlalchemy as sa
-from test.lib import testing
+from sqlalchemy import testing
 from sqlalchemy import MetaData, Integer, String, ForeignKey, func, \
     util, select
-from test.lib.schema import Table, Column
+from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import mapper, relationship, backref, \
-    class_mapper,  \
-    validates, aliased
+    class_mapper, CompositeProperty, \
+    validates, aliased, configure_mappers
 from sqlalchemy.orm import attributes, \
     composite, relationship, \
     Session
-from test.lib.testing import eq_
-from test.lib import fixtures
+from sqlalchemy.testing import eq_
+from sqlalchemy.testing import fixtures
 from test.orm import _fixtures
 
 
@@ -19,16 +19,15 @@ class PointTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('graphs', metadata,
-            Column('id', Integer, primary_key=True, 
+            Column('id', Integer, primary_key=True,
                                 test_needs_autoincrement=True),
             Column('name', String(30)))
 
         Table('edges', metadata,
-            Column('id', Integer, primary_key=True, 
+            Column('id', Integer, primary_key=True,
                                 test_needs_autoincrement=True),
-            Column('graph_id', Integer, 
-                                ForeignKey('graphs.id'), 
-                                nullable=False),
+            Column('graph_id', Integer,
+                                ForeignKey('graphs.id')),
             Column('x1', Integer),
             Column('y1', Integer),
             Column('x2', Integer),
@@ -115,6 +114,32 @@ class PointTest(fixtures.MappedTest):
         e = sess.query(Edge).get(g.edges[1].id)
         eq_(e.end, Point(18, 4))
 
+    def test_not_none(self):
+        Graph, Edge, Point = (self.classes.Graph,
+                                self.classes.Edge,
+                                self.classes.Point)
+
+        # current contract.   the composite is None
+        # when hasn't been populated etc. on a
+        # pending/transient object.
+        e1 = Edge()
+        assert e1.end is None
+        sess = Session()
+        sess.add(e1)
+
+        # however, once it's persistent, the code as of 0.7.3
+        # would unconditionally populate it, even though it's
+        # all None.  I think this usage contract is inconsistent,
+        # and it would be better that the composite is just
+        # created unconditionally in all cases.
+        # but as we are just trying to fix [ticket:2308] and
+        # [ticket:2309] without changing behavior we maintain
+        # that only "persistent" gets the composite with the
+        # Nones
+
+        sess.flush()
+        assert e1.end is not None
+
     def test_eager_load(self):
         Graph, Point = self.classes.Graph, self.classes.Point
 
@@ -147,25 +172,55 @@ class PointTest(fixtures.MappedTest):
         g = sess.query(Graph).first()
 
         assert sess.query(Edge).\
-                    filter(Edge.start==Point(3, 4)).one() is \
+                    filter(Edge.start == Point(3, 4)).one() is \
                     g.edges[0]
 
         assert sess.query(Edge).\
-                    filter(Edge.start!=Point(3, 4)).first() is \
+                    filter(Edge.start != Point(3, 4)).first() is \
                     g.edges[1]
 
         eq_(
-            sess.query(Edge).filter(Edge.start==None).all(), 
+            sess.query(Edge).filter(Edge.start == None).all(),
             []
         )
 
+    def test_comparator_aliased(self):
+        Graph, Edge, Point = (self.classes.Graph,
+                                self.classes.Edge,
+                                self.classes.Point)
+
+        sess = self._fixture()
+
+        g = sess.query(Graph).first()
+        ea = aliased(Edge)
+        assert sess.query(ea).\
+                    filter(ea.start != Point(3, 4)).first() is \
+                    g.edges[1]
+
+    def test_get_history(self):
+        Edge = self.classes.Edge
+        Point = self.classes.Point
+        from sqlalchemy.orm.attributes import get_history
+
+        e1 = Edge()
+        e1.start = Point(1,2)
+        eq_(
+            get_history(e1, 'start'),
+            ([Point(x=1, y=2)], (), [Point(x=None, y=None)])
+        )
+
+        eq_(
+            get_history(e1, 'end'),
+            ((), [Point(x=None, y=None)], ())
+        )
+
     def test_query_cols(self):
         Edge = self.classes.Edge
 
         sess = self._fixture()
 
         eq_(
-            sess.query(Edge.start, Edge.end).all(), 
+            sess.query(Edge.start, Edge.end).all(),
             [(3, 4, 5, 6), (14, 5, 2, 7)]
         )
 
@@ -179,7 +234,7 @@ class PointTest(fixtures.MappedTest):
         del e.end
         sess.flush()
         eq_(
-            sess.query(Edge.start, Edge.end).all(), 
+            sess.query(Edge.start, Edge.end).all(),
             [(3, 4, 5, 6), (14, 5, None, None)]
         )
 
@@ -225,9 +280,9 @@ class PrimaryKeyTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('graphs', metadata,
-            Column('id', Integer, primary_key=True, 
+            Column('id', Integer, primary_key=True,
                         test_needs_autoincrement=True),
-            Column('version_id', Integer, primary_key=True, 
+            Column('version_id', Integer, primary_key=True,
                                             nullable=True),
             Column('name', String(30)))
 
@@ -318,11 +373,11 @@ class DefaultsTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('foobars', metadata,
-            Column('id', Integer, primary_key=True, 
+            Column('id', Integer, primary_key=True,
                                 test_needs_autoincrement=True),
             Column('x1', Integer, default=2),
             Column('x2', Integer),
-            Column('x3', Integer, default=15),
+            Column('x3', Integer, server_default="15"),
             Column('x4', Integer)
         )
 
@@ -335,26 +390,29 @@ class DefaultsTest(fixtures.MappedTest):
 
         class FBComposite(cls.Comparable):
             def __init__(self, x1, x2, x3, x4):
-                self.x1 = x1
+                self.goofy_x1 = x1
                 self.x2 = x2
                 self.x3 = x3
                 self.x4 = x4
             def __composite_values__(self):
-                return self.x1, self.x2, self.x3, self.x4
+                return self.goofy_x1, self.x2, self.x3, self.x4
             __hash__ = None
             def __eq__(self, other):
-                return other.x1 == self.x1 and \
+                return other.goofy_x1 == self.goofy_x1 and \
                         other.x2 == self.x2 and \
                         other.x3 == self.x3 and \
                         other.x4 == self.x4
             def __ne__(self, other):
                 return not self.__eq__(other)
-
+            def __repr__(self):
+                return "FBComposite(%r, %r, %r, %r)" % (
+                    self.goofy_x1, self.x2, self.x3, self.x4
+                )
         mapper(Foobar, foobars, properties=dict(
-            foob=sa.orm.composite(FBComposite, 
-                                foobars.c.x1, 
-                                foobars.c.x2, 
-                                foobars.c.x3, 
+            foob=sa.orm.composite(FBComposite,
+                                foobars.c.x1,
+                                foobars.c.x2,
+                                foobars.c.x3,
                                 foobars.c.x4)
         ))
 
@@ -368,12 +426,12 @@ class DefaultsTest(fixtures.MappedTest):
         sess.add(f1)
         sess.flush()
 
-        assert f1.foob == FBComposite(2, 5, 15, None)
+        eq_(f1.foob, FBComposite(2, 5, 15, None))
 
         f2 = Foobar()
         sess.add(f2)
         sess.flush()
-        assert f2.foob == FBComposite(2, None, 15, None)
+        eq_(f2.foob, FBComposite(2, None, 15, None))
 
     def test_set_composite_values(self):
         Foobar, FBComposite = self.classes.Foobar, self.classes.FBComposite
@@ -384,22 +442,23 @@ class DefaultsTest(fixtures.MappedTest):
         sess.add(f1)
         sess.flush()
 
-        assert f1.foob == FBComposite(2, 5, 15, None)
+        eq_(f1.foob, FBComposite(2, 5, 15, None))
+
 
 class MappedSelectTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('descriptions', metadata,
-            Column('id', Integer, primary_key=True, 
+            Column('id', Integer, primary_key=True,
                             test_needs_autoincrement=True),
             Column('d1', String(20)),
             Column('d2', String(20)),
         )
 
         Table('values', metadata,
-            Column('id', Integer, primary_key=True, 
+            Column('id', Integer, primary_key=True,
                             test_needs_autoincrement=True),
-            Column('description_id', Integer, 
+            Column('description_id', Integer,
                             ForeignKey('descriptions.id'),
                             nullable=False),
             Column('v1', String(20)),
@@ -426,7 +485,7 @@ class MappedSelectTest(fixtures.MappedTest):
         desc_values = select(
             [values, descriptions.c.d1, descriptions.c.d2],
             descriptions.c.id == values.c.description_id
-        ).alias('descriptions_values') 
+        ).alias('descriptions_values')
 
         mapper(Descriptions, descriptions, properties={
             'values': relationship(Values, lazy='dynamic'),
@@ -438,7 +497,7 @@ class MappedSelectTest(fixtures.MappedTest):
         })
 
         mapper(Values, desc_values, properties={
-            'custom_values': composite(CustomValues, 
+            'custom_values': composite(CustomValues,
                                             desc_values.c.v1,
                                             desc_values.c.v2),
 
@@ -474,16 +533,16 @@ class MappedSelectTest(fixtures.MappedTest):
 class ManyToOneTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
-        Table('a', 
+        Table('a',
             metadata,
-            Column('id', Integer, primary_key=True, 
+            Column('id', Integer, primary_key=True,
                             test_needs_autoincrement=True),
             Column('b1', String(20)),
             Column('b2_id', Integer, ForeignKey('b.id'))
         )
 
         Table('b', metadata,
-            Column('id', Integer, primary_key=True, 
+            Column('id', Integer, primary_key=True,
                             test_needs_autoincrement=True),
             Column('data', String(20))
         )
@@ -541,7 +600,25 @@ class ManyToOneTest(fixtures.MappedTest):
         sess.commit()
 
         eq_(
-            sess.query(A).filter(A.c==C('a2b1', b2)).one(),
+            sess.query(A).filter(A.c == C('a2b1', b2)).one(),
+            a2
+        )
+
+    def test_query_aliased(self):
+        A, C, B = (self.classes.A,
+                                self.classes.C,
+                                self.classes.B)
+
+        sess = Session()
+        b1, b2 = B(data='b1'), B(data='b2')
+        a1 = A(c=C('a1b1', b1))
+        a2 = A(c=C('a2b1', b2))
+        sess.add_all([a1, a2])
+        sess.commit()
+
+        ae = aliased(A)
+        eq_(
+            sess.query(ae).filter(ae.c == C('a2b1', b2)).one(),
             a2
         )
 
@@ -549,7 +626,7 @@ class ConfigurationTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('edge', metadata,
-            Column('id', Integer, primary_key=True, 
+            Column('id', Integer, primary_key=True,
                                 test_needs_autoincrement=True),
             Column('x1', Integer),
             Column('y1', Integer),
@@ -621,4 +698,186 @@ class ConfigurationTest(fixtures.MappedTest):
         m.add_property('start', sa.orm.composite(Point, 'x1', 'y1'))
         m.add_property('end', sa.orm.composite(Point, 'x2', 'y2'))
 
-        self._test_roundtrip()
\ No newline at end of file
+        self._test_roundtrip()
+
+    def test_deferred(self):
+        edge, Edge, Point = (self.tables.edge,
+                                self.classes.Edge,
+                                self.classes.Point)
+        mapper(Edge, edge, properties={
+            'start':sa.orm.composite(Point, edge.c.x1, edge.c.y1,
+                                            deferred=True, group='s'),
+            'end': sa.orm.composite(Point, edge.c.x2, edge.c.y2,
+                                            deferred=True)
+        })
+        self._test_roundtrip()
+
+    def test_check_prop_type(self):
+        edge, Edge, Point = (self.tables.edge,
+                                self.classes.Edge,
+                                self.classes.Point)
+        mapper(Edge, edge, properties={
+            'start': sa.orm.composite(Point, (edge.c.x1,), edge.c.y1),
+        })
+        assert_raises_message(
+            sa.exc.ArgumentError,
+            # note that we also are checking that the tuple
+            # renders here, so the "%" operator in the string needs to
+            # apply the tuple also
+            r"Composite expects Column objects or mapped "
+            "attributes/attribute names as "
+            "arguments, got: \(Column",
+            configure_mappers
+        )
+
+class ComparatorTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
+    __dialect__ = 'default'
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('edge', metadata,
+            Column('id', Integer, primary_key=True,
+                                test_needs_autoincrement=True),
+            Column('x1', Integer),
+            Column('y1', Integer),
+            Column('x2', Integer),
+            Column('y2', Integer),
+        )
+
+    @classmethod
+    def setup_mappers(cls):
+        class Point(cls.Comparable):
+            def __init__(self, x, y):
+                self.x = x
+                self.y = y
+            def __composite_values__(self):
+                return [self.x, self.y]
+            def __eq__(self, other):
+                return isinstance(other, Point) and \
+                        other.x == self.x and \
+                        other.y == self.y
+            def __ne__(self, other):
+                return not isinstance(other, Point) or \
+                    not self.__eq__(other)
+
+        class Edge(cls.Comparable):
+            def __init__(self, start, end):
+                self.start = start
+                self.end = end
+
+            def __eq__(self, other):
+                return isinstance(other, Edge) and \
+                       other.id == self.id
+
+    def _fixture(self, custom):
+        edge, Edge, Point = (self.tables.edge,
+                                self.classes.Edge,
+                                self.classes.Point)
+
+        if custom:
+            class CustomComparator(sa.orm.CompositeProperty.Comparator):
+                def near(self, other, d):
+                    clauses = self.__clause_element__().clauses
+                    diff_x = clauses[0] - other.x
+                    diff_y = clauses[1] - other.y
+                    return diff_x * diff_x + diff_y * diff_y <= d * d
+
+            mapper(Edge, edge, properties={
+                'start': sa.orm.composite(Point, edge.c.x1, edge.c.y1,
+                                          comparator_factory=CustomComparator),
+                'end': sa.orm.composite(Point, edge.c.x2, edge.c.y2)
+            })
+        else:
+            mapper(Edge, edge, properties={
+                'start': sa.orm.composite(Point, edge.c.x1, edge.c.y1),
+                'end': sa.orm.composite(Point, edge.c.x2, edge.c.y2)
+            })
+
+    def test_comparator_behavior_default(self):
+        self._fixture(False)
+        self._test_comparator_behavior()
+
+    def test_comparator_behavior_custom(self):
+        self._fixture(True)
+        self._test_comparator_behavior()
+
+    def _test_comparator_behavior(self):
+        Edge, Point = (self.classes.Edge,
+                                self.classes.Point)
+
+        sess = Session()
+        e1 = Edge(Point(3, 4), Point(5, 6))
+        e2 = Edge(Point(14, 5), Point(2, 7))
+        sess.add_all([e1, e2])
+        sess.commit()
+
+        assert sess.query(Edge).\
+                    filter(Edge.start==Point(3, 4)).one() is \
+                    e1
+
+        assert sess.query(Edge).\
+                    filter(Edge.start!=Point(3, 4)).first() is \
+                    e2
+
+        eq_(
+            sess.query(Edge).filter(Edge.start==None).all(),
+            []
+        )
+
+    def test_default_comparator_factory(self):
+        self._fixture(False)
+        Edge = self.classes.Edge
+        start_prop = Edge.start.property
+
+        assert start_prop.comparator_factory is CompositeProperty.Comparator
+
+    def test_custom_comparator_factory(self):
+        self._fixture(True)
+        Edge, Point = (self.classes.Edge,
+                                self.classes.Point)
+
+        edge_1, edge_2 = Edge(Point(0, 0), Point(3, 5)), \
+                        Edge(Point(0, 1), Point(3, 5))
+
+        sess = Session()
+        sess.add_all([edge_1, edge_2])
+        sess.commit()
+
+        near_edges = sess.query(Edge).filter(
+            Edge.start.near(Point(1, 1), 1)
+        ).all()
+
+        assert edge_1 not in near_edges
+        assert edge_2 in near_edges
+
+        near_edges = sess.query(Edge).filter(
+            Edge.start.near(Point(0, 1), 1)
+        ).all()
+
+        assert edge_1 in near_edges and edge_2 in near_edges
+
+    def test_order_by(self):
+        self._fixture(False)
+        Edge = self.classes.Edge
+        s = Session()
+        self.assert_compile(
+            s.query(Edge).order_by(Edge.start, Edge.end),
+            "SELECT edge.id AS edge_id, edge.x1 AS edge_x1, "
+            "edge.y1 AS edge_y1, edge.x2 AS edge_x2, edge.y2 AS edge_y2 "
+            "FROM edge ORDER BY edge.x1, edge.y1, edge.x2, edge.y2"
+        )
+
+    def test_order_by_aliased(self):
+        self._fixture(False)
+        Edge = self.classes.Edge
+        s = Session()
+        ea = aliased(Edge)
+        self.assert_compile(
+            s.query(ea).order_by(ea.start, ea.end),
+            "SELECT edge_1.id AS edge_1_id, edge_1.x1 AS edge_1_x1, "
+            "edge_1.y1 AS edge_1_y1, edge_1.x2 AS edge_1_x2, "
+            "edge_1.y2 AS edge_1_y2 "
+            "FROM edge AS edge_1 ORDER BY edge_1.x1, edge_1.y1, "
+            "edge_1.x2, edge_1.y2"
+        )
+
