@@ -2,26 +2,36 @@
 Primary key changing capabilities and passive/non-passive cascading updates.
 
 """
-from test.lib.testing import eq_, ne_, \
+from __future__ import with_statement
+from sqlalchemy.testing import eq_, ne_, \
                         assert_raises, assert_raises_message
 import sqlalchemy as sa
-from test.lib import testing
+from sqlalchemy import testing
 from sqlalchemy import Integer, String, ForeignKey, Unicode
-from test.lib.schema import Table, Column
+from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import mapper, relationship, create_session, backref, Session
 from sqlalchemy.orm.session import make_transient
-from test.lib.testing import eq_
-from test.lib import fixtures
+from sqlalchemy.testing import eq_
+from sqlalchemy.testing import fixtures
 from test.orm import _fixtures
 
+def _backend_specific_fk_args():
+    if testing.requires.deferrable_fks.enabled:
+        fk_args = dict(deferrable=True, initially='deferred')
+    elif not testing.requires.on_update_cascade.enabled:
+        fk_args = dict()
+    else:
+        fk_args = dict(onupdate='cascade')
+    return fk_args
+
 class NaturalPKTest(fixtures.MappedTest):
+    # MySQL 5.5 on Windows crashes (the entire server, not the client)
+    # if you screw around with ON UPDATE CASCADE type of stuff.
+    __requires__ = 'skip_mysql_on_windows', 'on_update_or_deferrable_fks'
 
     @classmethod
     def define_tables(cls, metadata):
-        if testing.against('oracle'):
-            fk_args = dict(deferrable=True, initially='deferred')
-        else:
-            fk_args = dict(onupdate='cascade')
+        fk_args = _backend_specific_fk_args()
 
         users = Table('users', metadata,
             Column('username', String(50), primary_key=True),
@@ -30,20 +40,20 @@ class NaturalPKTest(fixtures.MappedTest):
 
         addresses = Table('addresses', metadata,
             Column('email', String(50), primary_key=True),
-            Column('username', String(50), 
+            Column('username', String(50),
                             ForeignKey('users.username', **fk_args)),
             test_needs_fk=True)
 
         items = Table('items', metadata,
             Column('itemname', String(50), primary_key=True),
-            Column('description', String(100)), 
+            Column('description', String(100)),
             test_needs_fk=True)
 
         users_to_items = Table('users_to_items', metadata,
-            Column('username', String(50), 
+            Column('username', String(50),
                                 ForeignKey('users.username', **fk_args),
                                 primary_key=True),
-            Column('itemname', String(50), 
+            Column('itemname', String(50),
                                 ForeignKey('items.itemname', **fk_args),
                                 primary_key=True),
             test_needs_fk=True)
@@ -124,8 +134,7 @@ class NaturalPKTest(fixtures.MappedTest):
         assert sess.query(User).get('ed').fullname == 'jack'
 
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_onetomany_passive(self):
         self._test_onetomany(True)
 
@@ -165,15 +174,15 @@ class NaturalPKTest(fixtures.MappedTest):
         def go():
             sess.flush()
         if not passive_updates:
-            # test passive_updates=False; 
+            # test passive_updates=False;
             #load addresses, update user, update 2 addresses
-            self.assert_sql_count(testing.db, go, 4) 
+            self.assert_sql_count(testing.db, go, 4)
         else:
             # test passive_updates=True; update user
-            self.assert_sql_count(testing.db, go, 1) 
+            self.assert_sql_count(testing.db, go, 1)
         sess.expunge_all()
         assert User(username='jack', addresses=[
-                                        Address(username='jack'), 
+                                        Address(username='jack'),
                                         Address(username='jack')]) == \
                             sess.query(User).get('jack')
 
@@ -186,15 +195,50 @@ class NaturalPKTest(fixtures.MappedTest):
         u1 = sess.query(User).get('fred')
         eq_(User(username='fred', fullname='jack'), u1)
 
-
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_manytoone_passive(self):
         self._test_manytoone(True)
 
     def test_manytoone_nonpassive(self):
         self._test_manytoone(False)
 
+    def test_manytoone_nonpassive_cold_mapping(self):
+        """test that the mapper-level m2o dependency processor
+        is set up even if the opposite side relationship
+        hasn't yet been part of a flush.
+
+        """
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+        with testing.db.begin() as conn:
+            conn.execute(users.insert(),
+                username='jack', fullname='jack'
+                )
+            conn.execute(addresses.insert(),
+                email='jack1', username='jack'
+            )
+            conn.execute(addresses.insert(),
+                email='jack2', username='jack'
+                )
+
+        mapper(User, users)
+        mapper(Address, addresses, properties={
+            'user': relationship(User,
+                    passive_updates=False)
+        })
+
+        sess = create_session()
+        u1 = sess.query(User).first()
+        a1, a2 = sess.query(Address).all()
+        u1.username = 'ed'
+
+        def go():
+            sess.flush()
+        self.assert_sql_count(testing.db, go, 3)
+
     def _test_manytoone(self, passive_updates):
         users, Address, addresses, User = (self.tables.users,
                                 self.classes.Address,
@@ -203,7 +247,7 @@ class NaturalPKTest(fixtures.MappedTest):
 
         mapper(User, users)
         mapper(Address, addresses, properties={
-            'user':relationship(User, passive_updates=passive_updates)
+            'user': relationship(User, passive_updates=passive_updates)
         })
 
         sess = create_session()
@@ -235,8 +279,8 @@ class NaturalPKTest(fixtures.MappedTest):
         eq_([Address(username='ed'), Address(username='ed')],
                 sess.query(Address).all())
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+
+    @testing.requires.on_update_cascade
     def test_onetoone_passive(self):
         self._test_onetoone(True)
 
@@ -282,8 +326,7 @@ class NaturalPKTest(fixtures.MappedTest):
         sess.expunge_all()
         eq_([Address(username='ed')], sess.query(Address).all())
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_bidirectional_passive(self):
         self._test_bidirectional(True)
 
@@ -298,7 +341,7 @@ class NaturalPKTest(fixtures.MappedTest):
 
         mapper(User, users)
         mapper(Address, addresses, properties={
-            'user':relationship(User, passive_updates=passive_updates,
+            'user': relationship(User, passive_updates=passive_updates,
                             backref='addresses')})
 
         sess = create_session()
@@ -341,15 +384,11 @@ class NaturalPKTest(fixtures.MappedTest):
                         sess.query(Address).all())
 
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_manytomany_passive(self):
         self._test_manytomany(True)
 
-    # mysqldb executemany() of the association table fails to 
-    # report the correct row count
-    @testing.fails_if(lambda: testing.against('mysql') 
-                            and not testing.against('+zxjdbc'))
+    @testing.requires.non_updating_cascade
     def test_manytomany_nonpassive(self):
         self._test_manytomany(False)
 
@@ -486,7 +525,7 @@ class ReversePKsTest(fixtures.MappedTest):
         session.add(a_editable)
         session.commit()
 
-        # do the switch in both directions - 
+        # do the switch in both directions -
         # one or the other should raise the error
         # based on platform dictionary ordering
         a_published.status = ARCHIVED
@@ -506,16 +545,15 @@ class ReversePKsTest(fixtures.MappedTest):
 
 
 class SelfReferentialTest(fixtures.MappedTest):
-    # mssql, mysql don't allow 
+    # mssql, mysql don't allow
     # ON UPDATE on self-referential keys
-    __unsupported_on__ = ('mssql','mysql') 
+    __unsupported_on__ = ('mssql', 'mysql')
+
+    __requires__ = 'on_update_or_deferrable_fks',
 
     @classmethod
     def define_tables(cls, metadata):
-        if testing.against('oracle'):
-            fk_args = dict(deferrable=True, initially='deferred')
-        else:
-            fk_args = dict(onupdate='cascade')
+        fk_args = _backend_specific_fk_args()
 
         Table('nodes', metadata,
               Column('name', String(50), primary_key=True),
@@ -581,8 +619,7 @@ class SelfReferentialTest(fixtures.MappedTest):
              for n in sess.query(Node).filter(
                  Node.name.in_(['n11', 'n12', 'n13']))])
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_many_to_one_passive(self):
         self._test_many_to_one(True)
 
@@ -593,8 +630,8 @@ class SelfReferentialTest(fixtures.MappedTest):
         Node, nodes = self.classes.Node, self.tables.nodes
 
         mapper(Node, nodes, properties={
-            'parentnode':relationship(Node, 
-                            remote_side=nodes.c.name, 
+            'parentnode':relationship(Node,
+                            remote_side=nodes.c.name,
                             passive_updates=passive)
             }
         )
@@ -616,12 +653,11 @@ class SelfReferentialTest(fixtures.MappedTest):
 
 
 class NonPKCascadeTest(fixtures.MappedTest):
+    __requires__ = 'skip_mysql_on_windows', 'on_update_or_deferrable_fks'
+
     @classmethod
     def define_tables(cls, metadata):
-        if testing.against('oracle'):
-            fk_args = dict(deferrable=True, initially='deferred')
-        else:
-            fk_args = dict(onupdate='cascade')
+        fk_args = _backend_specific_fk_args()
 
         Table('users', metadata,
             Column('id', Integer, primary_key=True,
@@ -646,8 +682,7 @@ class NonPKCascadeTest(fixtures.MappedTest):
         class Address(cls.Comparable):
             pass
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_onetomany_passive(self):
         self._test_onetomany(True)
 
@@ -681,7 +716,7 @@ class NonPKCascadeTest(fixtures.MappedTest):
         u1.username = 'ed'
         sess.flush()
         assert u1.addresses[0].username == 'ed'
-        eq_(sa.select([addresses.c.username]).execute().fetchall(), 
+        eq_(sa.select([addresses.c.username]).execute().fetchall(),
                     [('ed',), ('ed',)])
 
         sess.expunge_all()
@@ -693,14 +728,14 @@ class NonPKCascadeTest(fixtures.MappedTest):
         def go():
             sess.flush()
         if not passive_updates:
-            # test passive_updates=False; load addresses, 
+            # test passive_updates=False; load addresses,
             # update user, update 2 addresses
-            self.assert_sql_count(testing.db, go, 4) 
+            self.assert_sql_count(testing.db, go, 4)
         else:
              # test passive_updates=True; update user
             self.assert_sql_count(testing.db, go, 1)
         sess.expunge_all()
-        assert User(username='jack', 
+        assert User(username='jack',
                         addresses=[Address(username='jack'),
                                     Address(username='jack')]) == \
                     sess.query(User).get(u1.id)
@@ -714,7 +749,7 @@ class NonPKCascadeTest(fixtures.MappedTest):
         a1 = sess.query(Address).get(a1.id)
         eq_(a1.username, None)
 
-        eq_(sa.select([addresses.c.username]).execute().fetchall(), 
+        eq_(sa.select([addresses.c.username]).execute().fetchall(),
                         [(None,), (None,)])
 
         u1 = sess.query(User).get(u1.id)
@@ -727,17 +762,14 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
 
     @classmethod
     def define_tables(cls, metadata):
-        if testing.against('oracle'):
-            fk_args = dict(deferrable=True, initially='deferred')
-        else:
-            fk_args = dict(onupdate='cascade')
+        fk_args = _backend_specific_fk_args()
 
         Table('users', metadata,
             Column('username', String(50), primary_key=True),
             test_needs_fk=True)
 
         Table('addresses', metadata,
-                Column('username', String(50), 
+                Column('username', String(50),
                        ForeignKey('users.username', **fk_args),
                        primary_key=True
                        ),
@@ -753,13 +785,11 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         class Address(cls.Comparable):
             pass
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_onetomany_passive(self):
         self._test_onetomany(True)
 
-    # PG etc. need passive=True to allow PK->PK cascade
-    @testing.fails_on_everything_except('sqlite', 'oracle', '+zxjdbc')
+    @testing.requires.non_updating_cascade
     def test_onetomany_nonpassive(self):
         self._test_onetomany(False)
 
@@ -772,7 +802,7 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
     def _test_o2m_change(self, passive_updates):
         """Change the PK of a related entity to another.
 
-        "on update cascade" is not involved here, so the mapper has 
+        "on update cascade" is not involved here, so the mapper has
         to do the UPDATE itself.
 
         """
@@ -832,13 +862,11 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         u2.addresses.append(a1)
         sess.flush()
 
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE '
-                                'but requires referential integrity')
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_change_m2o_passive(self):
         self._test_change_m2o(True)
 
-    @testing.fails_on_everything_except('sqlite', 'oracle', '+zxjdbc')
+    @testing.requires.non_updating_cascade
     def test_change_m2o_nonpassive(self):
         self._test_change_m2o(False)
 
@@ -922,7 +950,7 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         sess.add(u2)
         sess.add(a2)
 
-        from test.lib.assertsql import CompiledSQL
+        from sqlalchemy.testing.assertsql import CompiledSQL
 
         # test that the primary key columns of addresses are not
         # being updated as well, since this is a row switch.
@@ -940,7 +968,7 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
     def _test_onetomany(self, passive_updates):
         """Change the PK of a related entity via foreign key cascade.
 
-        For databases that require "on update cascade", the mapper 
+        For databases that require "on update cascade", the mapper
         has to identify the row by the new value, not the old, when
         it does the update.
 
@@ -964,7 +992,7 @@ class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
         sess.flush()
         eq_(a1.username, 'ed')
         eq_(a2.username, 'ed')
-        eq_(sa.select([addresses.c.username]).execute().fetchall(), 
+        eq_(sa.select([addresses.c.username]).execute().fetchall(),
                 [('ed',), ('ed',)])
 
         u1.username = 'jack'
@@ -981,14 +1009,13 @@ class JoinedInheritanceTest(fixtures.MappedTest):
     """Test cascades of pk->pk/fk on joined table inh."""
 
     # mssql doesn't allow ON UPDATE on self-referential keys
-    __unsupported_on__ = ('mssql',) 
+    __unsupported_on__ = ('mssql',)
+
+    __requires__ = 'skip_mysql_on_windows',
 
     @classmethod
     def define_tables(cls, metadata):
-        if testing.against('oracle'):
-            fk_args = dict(deferrable=True, initially='deferred')
-        else:
-            fk_args = dict(onupdate='cascade')
+        fk_args = _backend_specific_fk_args()
 
         Table('person', metadata,
             Column('name', String(50), primary_key=True),
@@ -999,13 +1026,13 @@ class JoinedInheritanceTest(fixtures.MappedTest):
             Column('name', String(50), ForeignKey('person.name', **fk_args),
                                         primary_key=True),
             Column('primary_language', String(50)),
-            Column('boss_name', String(50), 
+            Column('boss_name', String(50),
                                     ForeignKey('manager.name', **fk_args)),
                                     test_needs_fk=True
         )
 
         Table('manager', metadata,
-            Column('name', String(50), 
+            Column('name', String(50),
                                     ForeignKey('person.name', **fk_args),
                                     primary_key=True),
             Column('paperwork', String(50)),
@@ -1021,24 +1048,20 @@ class JoinedInheritanceTest(fixtures.MappedTest):
         class Manager(Person):
             pass
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_pk_passive(self):
         self._test_pk(True)
 
-    # PG etc. need passive=True to allow PK->PK cascade
-    @testing.fails_on_everything_except('sqlite', 'oracle', '+zxjdbc')
+    @testing.requires.non_updating_cascade
     def test_pk_nonpassive(self):
         self._test_pk(False)
 
-    @testing.fails_on('sqlite', 'sqlite doesnt support ON UPDATE CASCADE')
-    @testing.fails_on('oracle', 'oracle doesnt support ON UPDATE CASCADE')
+    @testing.requires.on_update_cascade
     def test_fk_passive(self):
         self._test_fk(True)
 
     # PG etc. need passive=True to allow PK->PK cascade
-    @testing.fails_on_everything_except('sqlite', 'mysql+zxjdbc', 'oracle',
-                                                'postgresql+zxjdbc')
+    @testing.requires.non_updating_cascade
     def test_fk_nonpassive(self):
         self._test_fk(False)
 
@@ -1050,12 +1073,12 @@ class JoinedInheritanceTest(fixtures.MappedTest):
                                 self.classes.Engineer,
                                 self.tables.engineer)
 
-        mapper(Person, person, polymorphic_on=person.c.type, 
+        mapper(Person, person, polymorphic_on=person.c.type,
                 polymorphic_identity='person',
                 passive_updates=passive_updates)
         mapper(Engineer, engineer, inherits=Person,
             polymorphic_identity='engineer', properties={
-            'boss':relationship(Manager, 
+            'boss':relationship(Manager,
                         primaryjoin=manager.c.name==engineer.c.boss_name,
                         passive_updates=passive_updates
                         )
@@ -1080,12 +1103,12 @@ class JoinedInheritanceTest(fixtures.MappedTest):
                                 self.classes.Engineer,
                                 self.tables.engineer)
 
-        mapper(Person, person, polymorphic_on=person.c.type, 
+        mapper(Person, person, polymorphic_on=person.c.type,
                 polymorphic_identity='person',
                         passive_updates=passive_updates)
         mapper(Engineer, engineer, inherits=Person,
                         polymorphic_identity='engineer', properties={
-            'boss':relationship(Manager, 
+            'boss':relationship(Manager,
                         primaryjoin=manager.c.name==engineer.c.boss_name,
                         passive_updates=passive_updates
                         )
