@@ -1,25 +1,22 @@
 # coding: utf-8
 """Tests unitofwork operations."""
 
-from test.lib.testing import eq_, assert_raises, assert_raises_message
+from sqlalchemy.testing import eq_, assert_raises, assert_raises_message
 import datetime
-import operator
 from sqlalchemy.orm import mapper as orm_mapper
 
 import sqlalchemy as sa
 from sqlalchemy import Integer, String, ForeignKey, literal_column, event
-from test.lib import engines, testing, pickleable
-from test.lib.schema import Table
-from test.lib.schema import Column
+from sqlalchemy.testing import engines
+from sqlalchemy import testing
+from sqlalchemy.testing.schema import Table
+from sqlalchemy.testing.schema import Column
 from sqlalchemy.orm import mapper, relationship, create_session, \
-    column_property, attributes, Session, reconstructor, object_session
-from test.lib.testing import eq_, ne_
-from test.lib.util import gc_collect
-from test.lib import fixtures
+    column_property, Session, exc as orm_exc
+from sqlalchemy.testing import fixtures
 from test.orm import _fixtures
-from test.lib import fixtures
-from test.lib.assertsql import AllOf, CompiledSQL
-import gc
+from sqlalchemy.testing.assertsql import AllOf, CompiledSQL
+from sqlalchemy import testing, util
 
 class UnitOfWorkTest(object):
     pass
@@ -162,6 +159,9 @@ class UnicodeSchemaTest(fixtures.MappedTest):
 
     @testing.fails_on('mssql+pyodbc',
                       'pyodbc returns a non unicode encoding of the results description.')
+    @testing.skip_if(lambda: util.pypy,
+            "pypy/sqlite3 reports unicode cursor.description "
+            "incorrectly pre 2.2, workaround applied in 0.9")
     def test_mapping(self):
         t2, t1 = self.tables.t2, self.tables.t1
 
@@ -201,6 +201,9 @@ class UnicodeSchemaTest(fixtures.MappedTest):
 
     @testing.fails_on('mssql+pyodbc',
                       'pyodbc returns a non unicode encoding of the results description.')
+    @testing.skip_if(lambda: util.pypy,
+            "pypy/sqlite3 reports unicode cursor.description "
+            "incorrectly pre 2.2, workaround applied in 0.9")
     def test_inheritance_mapping(self):
         t2, t1 = self.tables.t2, self.tables.t1
 
@@ -567,6 +570,31 @@ class PassiveDeletesTest(fixtures.MappedTest):
         mapper(MyClass, mytable)
         assert_raises(sa.exc.SAWarning, sa.orm.configure_mappers)
 
+class BatchDeleteIgnoresRowcountTest(fixtures.DeclarativeMappedTest):
+    __requires__ = ('foreign_keys',)
+    @classmethod
+    def setup_classes(cls):
+        class A(cls.DeclarativeBasic):
+            __tablename__ = 'A'
+            __table_args__ = dict(test_needs_fk=True)
+            id = Column(Integer, primary_key=True)
+            parent_id = Column(Integer, ForeignKey('A.id', ondelete='CASCADE'))
+
+    def test_delete_both(self):
+        A = self.classes.A
+        session = Session(testing.db)
+
+        a1, a2 = A(id=1), A(id=2, parent_id=1)
+
+        session.add_all([a1, a2])
+        session.flush()
+
+        session.delete(a1)
+        session.delete(a2)
+
+        # no issue with multi-row count here
+        session.flush()
+
 class ExtraPassiveDeletesTest(fixtures.MappedTest):
     __requires__ = ('foreign_keys',)
 
@@ -595,19 +623,25 @@ class ExtraPassiveDeletesTest(fixtures.MappedTest):
 
     def test_assertions(self):
         myothertable, MyOtherClass = self.tables.myothertable, self.classes.MyOtherClass
+        mytable, MyClass = self.tables.mytable, self.classes.MyClass
 
+        mapper(MyClass, mytable, properties={
+            'foo': relationship(MyOtherClass,
+                                    passive_deletes='all',
+                                    cascade="all")
+            })
         mapper(MyOtherClass, myothertable)
+
         assert_raises_message(
             sa.exc.ArgumentError,
-            "Can't set passive_deletes='all' in conjunction with 'delete' "
+            "On MyClass.foo, can't set passive_deletes='all' in conjunction with 'delete' "
             "or 'delete-orphan' cascade",
-            relationship, MyOtherClass,
-                                    passive_deletes='all',
-                                    cascade="all"
+            sa.orm.configure_mappers
         )
 
     def test_extra_passive(self):
-        myothertable, MyClass, MyOtherClass, mytable = (self.tables.myothertable,
+        myothertable, MyClass, MyOtherClass, mytable = (
+                                self.tables.myothertable,
                                 self.classes.MyClass,
                                 self.classes.MyOtherClass,
                                 self.tables.mytable)
@@ -918,13 +952,13 @@ class DefaultTest(fixtures.MappedTest):
 class ColumnPropertyTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
-        Table('data', metadata, 
+        Table('data', metadata,
             Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('a', String(50)),
             Column('b', String(50))
             )
 
-        Table('subdata', metadata, 
+        Table('subdata', metadata,
             Column('id', Integer, ForeignKey('data.id'), primary_key=True),
             Column('c', String(50)),
             )
@@ -946,7 +980,7 @@ class ColumnPropertyTest(fixtures.MappedTest):
         Data, data = self.classes.Data, self.tables.data
 
         mapper(Data, data, properties={
-            'aplusb':column_property(data.c.a + literal_column("' '") + data.c.b, 
+            'aplusb':column_property(data.c.a + literal_column("' '") + data.c.b,
                         expire_on_flush=False)
         })
         self._test(False)
@@ -1600,7 +1634,6 @@ class ManyToOneTest(_fixtures.FixtureTest):
             session.add(a)
 
         session.flush()
-
         objects[2].email_address = 'imnew@foo.bar'
         objects[3].user = User()
         objects[3].user.name = 'imnewlyadded'
@@ -1826,8 +1859,8 @@ class ManyToManyTest(_fixtures.FixtureTest):
         k.name = 'yellow'
         objects[5].keywords.append(k)
         self.assert_sql_execution(
-            testing.db, 
-            session.flush, 
+            testing.db,
+            session.flush,
             AllOf(
                 CompiledSQL("UPDATE items SET description=:description "
                  "WHERE items.id = :items_id",
@@ -1849,8 +1882,8 @@ class ManyToManyTest(_fixtures.FixtureTest):
         dkid = objects[5].keywords[1].id
         del objects[5].keywords[1]
         self.assert_sql_execution(
-            testing.db, 
-            session.flush, 
+            testing.db,
+            session.flush,
             CompiledSQL("DELETE FROM item_keywords "
                      "WHERE item_keywords.item_id = :item_id AND "
                      "item_keywords.keyword_id = :keyword_id",
@@ -2036,8 +2069,8 @@ class SaveTest2(_fixtures.FixtureTest):
         session.add_all(fixture())
 
         self.assert_sql_execution(
-            testing.db, 
-            session.flush, 
+            testing.db,
+            session.flush,
             CompiledSQL("INSERT INTO users (name) VALUES (:name)",
              {'name': 'u1'}),
             CompiledSQL("INSERT INTO users (name) VALUES (:name)",
@@ -2441,7 +2474,7 @@ class PartialNullPKTest(fixtures.MappedTest):
         t1 = s.query(T1).first()
         t1.col2 = 5
         assert_raises_message(
-            sa.exc.FlushError,
+            orm_exc.FlushError,
             "Can't update table using NULL for primary key value",
             s.commit
         )
@@ -2454,7 +2487,7 @@ class PartialNullPKTest(fixtures.MappedTest):
         t1 = s.query(T1).first()
         t1.col3 = 'hi'
         assert_raises_message(
-            sa.exc.FlushError,
+            orm_exc.FlushError,
             "Can't update table using NULL for primary key value",
             s.commit
         )
@@ -2467,7 +2500,7 @@ class PartialNullPKTest(fixtures.MappedTest):
         t1 = s.query(T1).first()
         s.delete(t1)
         assert_raises_message(
-            sa.exc.FlushError,
+            orm_exc.FlushError,
             "Can't delete from table using NULL for primary key value",
             s.commit
         )
@@ -2477,9 +2510,19 @@ class PartialNullPKTest(fixtures.MappedTest):
         s = Session()
         s.add(T1(col1=None, col2=None))
         assert_raises_message(
-            sa.exc.FlushError,
+            orm_exc.FlushError,
             r"Instance \<T1 at .+?\> has a NULL "
             "identity key.  If this is an auto-generated value, "
             "check that the database table allows generation ",
             s.commit
         )
+
+    def test_dont_complain_if_no_update(self):
+        T1 = self.classes.T1
+        s = Session()
+        t = T1(col1="1", col2=None)
+        s.add(t)
+        s.commit()
+
+        t.col1 = "1"
+        s.commit()
